/*****************************************************************************
*
*       (c) Copyright Kofax Image Products, 1990-1992.  All rights reserved.
*               Unauthorized use, duplication, or distribution is strictly prohibited.
*
******************************************************************************
*
*                File:  kci.h
*
*         Purpose:      Provide interface definitions for the KCI (Kofax Chucker
*                               Interface) module.  
*
*       $Revision:   1.6  $
*               $Date:   28 Jul 1992 11:37:00  $
*
*   $NoAuthor:$
*
*      $NoLog:$
*
*****************************************************************************/

#if !defined(kci_h)
#define kci_h

typedef struct tag_DPCB FAR             *LPDPCB;        /* Pointer to a DPCB */

typedef LPVOID FAR                              *LPADDR;        /* Pointer to an address */

/***** Defines for WinFlags machine status word returned by KCIGetWinFlags */

#define WF_PMODE                0x0001
#define WF_CPU286               0x0002
#define WF_CPU386               0x0004
#define WF_CPU486               0x0008
#define WF_STANDARD             0x0010
#define WF_WIN286               0x0010
#define WF_ENHANCED             0x0020
#define WF_WIN386               0x0020
#define WF_CPU086               0x0040
#define WF_CPU186               0x0080
#define WF_LARGEFRAME   0x0100
#define WF_SMALLFRAME   0x0200
#define WF_80x87                0x0400

/***** Application support functions (Windows only) */

#if defined(WIN)

/*****************************************************************************
* Function:     KCIGetWinFlags
* Purpose:      Get the Windows information flags.
* Returns:      Flag word on success
*                       0xffff on failure
*****************************************************************************/

WORD FAR PASCAL KCIGetWinFlags(void);

/*****************************************************************************
* Function:     KCISegToSelector
* Purpose:      Convert a segment/offset address to a protected-mode selector/offset
*                       address.
* Returns:      Protected-mode selector/offset on success
*                       On failure, high word will be zero and low word will be the status.
*****************************************************************************/

DWORD FAR PASCAL KCISegToSelector
( 
        WORD                    wSegment,               /* (I) The address' segment */
        WORD                    wOffset                 /* (I) The address' offset */
);

#endif /* defined(WIN) */

/***** Chucker support functions */

/*****************************************************************************
* Function:     KCIOpenChuck
* Purpose:      Open the specified chucker if it exists.
* Returns:      SUCCESS on success
*                       error code on failure
*****************************************************************************/

KERR_ERR_ENUM FAR PASCAL KCIOpenChuck
(
        DWORD                   dwChuckerId,    /* (I) ID of chucker to locate */
        LPADDR                  lpfnChucker             /* (O) Returns chucker address */
);

#if defined(WIN)

/*****************************************************************************
* Function:     KCICloseChuck
* Purpose:      Close the specified chucker.
* Returns:      SUCCESS on success
*                       error code on failure.
*****************************************************************************/

KERR_ERR_ENUM FAR PASCAL KCICloseChuck
(
        DWORD                   dwChuckerId             /* (I) ID of chucker to close */
);

#else   /* not WIN */
        #define KCICloseChuck(dwChuckerId)      SUCCESS
#endif

#endif /* kci_h */
