/*****************************************************************************
*
*       (c) Copyright Kofax Image Products, 1990-1992.  All rights reserved.
*               Unauthorized use, duplication, or distribution is strictly prohibited.
*
******************************************************************************
*
*                File:  kerr.h
*
*         Purpose:      Provide Kofax error/status return symbols
*
*       $Revision:   1.32  $
*               $Date:   09 Sep 1992 10:53:20  $
*
*   $NoAuthor:$
*
*      $NoLog:$
*
*****************************************************************************/

#if !defined(kerr_h)
#define kerr_h

/*****************************************************************************
* Generate a valid status ID encoding module ID and the status ID.
* Note that high bit is always set (the resulting status is always negative).
* Note use of signed int rather than hex (to set high bit), enum requires this.
*****************************************************************************/

#define KERR_ERR(ModId,StatusId)        (-32768 + ((ModId) << 8) + (StatusId))

/*****************************************************************************
* KERR_STATUS_MOD_ID takes a status value and returns the module ID.  KERR_MOD
* is used by KERR_STATUS_MOD_ID and should not be used elsewhere.
*****************************************************************************/

#define KERR_STATUS_MOD_ID(Status)      (KERR_GET_MOD((((Status) >> 8)) & 0x7f))
#define KERR_GET_MOD(ModId)                     (((ModId) > 0x3f) ? KIPP_MOD : (ModId))

/*****************************************************************************
* Module ID's.  These can range from 0x01 to 0x3f.  0x00 is not recommended.
* 0x40 to 0x7f is reserved for mapping to existing KIPP errors.  These are 
* used in building status codes.
*****************************************************************************/

#define KIPP_MOD                0x7f    /* Kofax Image Processing Platform (KIPP) mod */
#define KVE_MOD                 0x01    /* Kofax Virtual Engine (KVE) module */
#define KIE_MOD                 0x02    /* KIE module */
#define KVE_USR_MOD             0x03    /* KVE user module */
#define KERR_MOD                0x04    /* KERR (Kofax Error) module */
#define KUT_MOD                 0x05    /* Kofax Utilities module */
#define KSE_MOD                 0x06    /* KSE-specific (TMSFAX) module */
#define KHE_MOD                 0x07    /* KHE (Kofax Hardwar Engine) module */
#define HE_MOD                  0x08    /* HE (Hardware Engine) module */
#define KOS_MOD                 0x09    /* KOS (Kofax OS-Specific module) */

#ifndef SUCCESS
        #define SUCCESS                 0x0000
#endif

typedef enum {


/* ========================= */
/* =====  FROM KVE.H   ===== */
/* ========================= */

    KVEERR_NOT_IMPLEMENTED          = KERR_ERR( KVE_MOD, 0x01 ),           /* Unimplemented feature */
    KVEERR_BAD_PARAM_ID             = KERR_ERR( KVE_MOD, 0x02 ),           /* Invalid parameter ID */
    KVEERR_BAD_PARAM_VALUE          = KERR_ERR( KVE_MOD, 0x03 ),           /* Invalid paramter value */
    KVEERR_BAD_PARAM_TYPE           = KERR_ERR( KVE_MOD, 0x04 ),           /* Invalid parameter type */
    KVEERR_READ_ONLY_PARAM          = KERR_ERR( KVE_MOD, 0x05 ),           /* Paramter is read-only */
    KVEERR_BAD_PARAM_SIZE           = KERR_ERR( KVE_MOD, 0x06 ),           /* Incorrect parameter size */
    KVEERR_MULTIPLY_OVERFLOW        = KERR_ERR( KVE_MOD, 0x07 ),           /* Multiply overflow */
    KVEERR_BAD_BUFFER_SIZE          = KERR_ERR( KVE_MOD, 0x08 ),           /* Incorrect buffer size */
    KVEERR_BAD_DPCB_SIZE            = KERR_ERR( KVE_MOD, 0x09 ),           /* Incorrect DPCB size */
    KVEERR_NULL_POINTER             = KERR_ERR( KVE_MOD, 0x0A ),           /* Unexpected null pointer */
    KVEERR_ZERO_LENGTH              = KERR_ERR( KVE_MOD, 0x0B ),           /* TBD */
    KVEERR_TRUNCATED                = KERR_ERR( KVE_MOD, 0x0C ),           /* TBD */
    KVEERR_NOT_SUPPORTED            = KERR_ERR( KVE_MOD, 0x0D ),           /* Unsupported feature */
    KVEERR_BAD_STRIP_PITCH          = KERR_ERR( KVE_MOD, 0x0E ),           /* Bad strip pitch */
    KVEERR_BAD_STRIP_LENGTH         = KERR_ERR( KVE_MOD, 0x0F ),           /* Bad strip length */
    KVEERR_NO_AVAILABLE_HOST_INS    = KERR_ERR( KVE_MOD, 0x10 ),           /* No available Host In's */
    KVEERR_NO_COMPLETED_HOST_INS    = KERR_ERR( KVE_MOD, 0x11 ),           /* No completed Host In's */
    KVEERR_NO_AVAILABLE_HOST_OUTS   = KERR_ERR( KVE_MOD, 0x12 ),           /* No available Host Out's */
    KVEERR_NO_COMPLETED_HOST_OUTS   = KERR_ERR( KVE_MOD, 0x13 ),           /* No completed Host Out's */
    KVEERR_EMPTY_BUFFER             = KERR_ERR( KVE_MOD, 0x14 ),           /* Buffer is empty */
    KVEERR_BAD_DATA_PITCH           = KERR_ERR( KVE_MOD, 0x15 ),           /* Bad data pitch */
    KVEERR_NULL_FUNC_PTR            = KERR_ERR( KVE_MOD, 0x16 ),           /* Function pointer is NULL */

    KVEERR_SOFTWARE_ERROR           = KERR_ERR( KVE_MOD, 0x20 ),           /* TBD */
    KVEERR_NO_GLOBAL_MEMORY         = KERR_ERR( KVE_MOD, 0x21 ),           /* No global memory */
    KVEERR_GLOBAL_LOCK_FAILED       = KERR_ERR( KVE_MOD, 0x22 ),           /* Global memory lock failed */

    KVEERR_BAD_CXT_BUFFER_DESC      = KERR_ERR( KVE_MOD, 0x30 ),           /* Bad context buffer descriptor */
    KVEERR_BAD_CXT_SIZE             = KERR_ERR( KVE_MOD, 0x31 ),           /* Size of given context is incorrect */
    KVEERR_MISSING_CXT_VALUE        = KERR_ERR( KVE_MOD, 0x32 ),           /* Context value required for computation not set */
    KVEERR_BAD_HANDLE               = KERR_ERR( KVE_MOD, 0x33 ),           /* Memory handle type is unsupported */
    KVEERR_BAD_CXT                  = KERR_ERR( KVE_MOD, 0x34 ),           /* TBD */
    KVEERR_UNINITIALIZED_CXT        = KERR_ERR( KVE_MOD, 0x35 ),           /* Context invalid or not initialized */

    KVEERR_DRIVER_NOT_FOUND         = /* KERR_ERR( KVE_MOD, 0x40 ) */ -29, /* Driver not found */
    KVEERR_MORE_THAN_ONE_DRIVER     = KERR_ERR( KVE_MOD, 0x41 ),           /* More than one driver detected */
    KVEERR_DRIVER_ERROR             = KERR_ERR( KVE_MOD, 0x42 ),           /* Driver error */
    KVEERR_GENERIC_DRIVER_ERROR     = KERR_ERR( KVE_MOD, 0x43 ),           /* Generic driver error */

    KVEERR_ALREADY_LOCKED           = KERR_ERR( KVE_MOD, 0x44 ),           /* Engine already locked */
    KVEERR_NOT_LOCKED               = KERR_ERR( KVE_MOD, 0x45 ),           /* Engine not locked */
    KVEERR_LOCKED_BY_ANOTHER        = KERR_ERR( KVE_MOD, 0x46 ),           /* Engine locked by another */
    KVEERR_NO_ENGINE_AVAIL          = KERR_ERR( KVE_MOD, 0x47 ),           /* No engine available */
    KVEERR_BAD_ENGINE               = KERR_ERR( KVE_MOD, 0x48 ),           /* Invalid engine */

    KVEERR_POWER_OFF                = /* KERR_ERR( KVE_MOD, 0x50 ) */ -34, /* Device: power off */
    KVEERR_TIMEOUT                  = /* KERR_ERR( KVE_MOD, 0x51 ) */ -28, /* Device: timeout */
    KVEERR_NO_PAPER                 = /* KERR_ERR( KVE_MOD, 0x52 ) */ -37, /* Device: no paper */
    KVEERR_PAPER_JAM                = /* KERR_ERR( KVE_MOD, 0x53 ) */ -38, /* Device: paper jam */
    KVEERR_STOP_BUTTON              = /* KERR_ERR( KVE_MOD, 0x54 ) */ -39, /* TBD */
    KVEERR_DEVICE_FAILURE           = /* KERR_ERR( KVE_MOD, 0x55 ) */ -40, /* Device: device failure */
    KVEERR_DATA_UNDERRUN            = /* KERR_ERR( KVE_MOD, 0x56 ) */ -44, /* Data underrun */
    KVEERR_DEVICE_NOT_READY         = /* KERR_ERR( KVE_MOD, 0x57 ) */ -35, /* Device: device not ready */
    KVEERR_TONER_LOW                = /* KERR_ERR( KVE_MOD, 0x58 ) */ -36, /* Device: toner low */
    KVEERR_NOT_CONFIGURED           = /* KERR_ERR( KVE_MOD, 0x59 ) */ -42, /* TBD */
    KVEERR_COMPRESSED_DATA_ERROR    = /* KERR_ERR( KVE_MOD, 0x5A ) */ -22, /* TBD */
    KVEERR_NO_AVAIL_ADAPTER_BUFS    = KERR_ERR( KVE_MOD, 0x5B ),           /* Device: no available adapter buffers */

    KVEERR_BAD_CBVER                = KERR_ERR( KVE_MOD, 0x5C ),           /* Driver: KHE/Chucker version mismatch */

        /*      Operation status -- signify occurance of events during an operation */

    KVEERR_OP_OUTPUT_AVAILABLE      = KERR_ERR( KVE_MOD, 0x61 ),           /* Output is available */
    KVEERR_OP_OPERATION_COMPLETE    = KERR_ERR( KVE_MOD, 0x62 ),           /* Operation is compete */
    KVEERR_OP_NO_OUTPUT_AVAILABLE   = KERR_ERR( KVE_MOD, 0x63 ),           /* No output is available */
    KVEERR_OP_OUT_ALREADY_CLAIMED   = KERR_ERR( KVE_MOD, 0x64 ),           /* Output has already been claimed */
    KVEERR_OP_OUTPUT_NOT_CLAIMED    = KERR_ERR( KVE_MOD, 0x65 ),           /* Output has not been claimed */
    KVEERR_OP_WAITING_FOR_OUTPUT    = KERR_ERR( KVE_MOD, 0x66 ),           /* Waiting for output */
    KVEERR_OP_WAITING_FOR_INPUT     = KERR_ERR( KVE_MOD, 0x67 ),           /* Waiting for input */
    KVEERR_OP_OPERATION_NOT_RUNNING = KERR_ERR( KVE_MOD, 0x68 ),           /* Operation is not running */

        /* More KSE/KHE common errors */

    KVEERR_UNSUPPORTED_HANDLE       = KERR_ERR( KVE_MOD, 0x70 ),           /* Unsupported memory handle type */
    KVEERR_NONZERO_PTR_OFFSET       = KERR_ERR( KVE_MOD, 0x71 ),           /* Provided pointer does not have zero offset */
    KVEERR_UNSUPPORTED_ROTATION     = KERR_ERR( KVE_MOD, 0x72 ),           /* Invalid rotation */
    KVEERR_IN_BUF_TOO_SMALL         = KERR_ERR( KVE_MOD, 0x73 ),           /* Input buffer is too small */
    KVEERR_OUT_BUF_TOO_SMALL        = KERR_ERR( KVE_MOD, 0x74 ),           /* Output buffer is too small */
    KVEERR_UNSUPPORTED_COMPRESSION  = KERR_ERR( KVE_MOD, 0x75 ),           /* Unsupported compression format */
    KVEERR_UNKNOWN_KSE_STATUS       = KERR_ERR( KVE_MOD, 0x76 ),           /* Unknown Compression/Expansion error */

    KVEERR_BAD_RAWIMAGE             = KERR_ERR( KVE_MOD, 0x77 ),           /* Bad context value: RawImage */
    KVEERR_BAD_CODESDESC            = KERR_ERR( KVE_MOD, 0x78 ),           /* Bad context value: CodesDesc */
    KVEERR_BAD_OUTPUTRECT           = KERR_ERR( KVE_MOD, 0x79 ),           /* Bad context value: OutputRect */
    KVEERR_NULL_CXT                 = KERR_ERR( KVE_MOD, 0x7A ),           /* Bad argument: NULL context */
    KVEERR_NULL_INPUT_FUNC          = KERR_ERR( KVE_MOD, 0x7B ),           /* Bad argument: NULL input function */
    KVEERR_NULL_START_POINT         = KERR_ERR( KVE_MOD, 0x7C ),           /* Bad argument: NULL start point */
    KVEERR_NULL_DPCB                = KERR_ERR( KVE_MOD, 0x7D ),           /* Bad argument: NULL DPCB */
    KVEERR_NULL_BUFDESC             = KERR_ERR( KVE_MOD, 0x7E ),           /* Bad argument: NULL BufDesc */
    KVEERR_TOO_FEW_DPCBS            = KERR_ERR( KVE_MOD, 0x7F ),           /* Bad argument: Too few DPCB's */
    KVEERR_NULL_RAWSTRIP            = KERR_ERR( KVE_MOD, 0x80 ),           /* Bad argument: NULL RawStrip */
    KVEERR_BAD_OUTPUT_STRIP         = KERR_ERR( KVE_MOD, 0x81 ),           /* Bad argument: Invalid OutputStrip */
    KVEERR_UNSUPPORTED_ROT_FMT      = KERR_ERR( KVE_MOD, 0x82 ),           /* Invalid rotation format (collated vs. uncollated) */
    KVEERR_WRONG_BUF_COUNT          = KERR_ERR( KVE_MOD, 0x83 ),           /* Bad argument: Wrong number of buffers supplied */
    KVEERR_STORAGE_FILTER_FAIL      = KERR_ERR( KVE_MOD, 0x84 ),           /* Storage filter failure */
    KVEERR_BAD_START_PT_OFFSETS     = KERR_ERR( KVE_MOD, 0x85 ),           /* Invalid start point offsets calculated */
    KVEERR_BAD_SCALE                            = KERR_ERR( KVE_MOD, 0x86 ),           /* Invalid scale factors */

        /*      OS/2 errors encountered while access chucker */

    KVEERR_DEVIO_FAIL                           = KERR_ERR( KVE_MOD, 0x90 ),           /* TBD */
    KVEERR_DEVIO_BAD_DRIVER_LEVEL       = KERR_ERR( KVE_MOD, 0x91 ),           /* TBD */
    KVEERR_DEVIO_INVALID_CATEGORY       = KERR_ERR( KVE_MOD, 0x92 ),           /* TBD */
    KVEERR_DEVIO_INVALID_DRIVE          = KERR_ERR( KVE_MOD, 0x93 ),           /* TBD */
    KVEERR_DEVIO_INVALID_FUNC           = KERR_ERR( KVE_MOD, 0x94 ),           /* TBD */
    KVEERR_DEVIO_INVALID_HANDLE         = KERR_ERR( KVE_MOD, 0x95 ),           /* TBD */
    KVEERR_DEVIO_PROT_VIOLATION         = KERR_ERR( KVE_MOD, 0x96 ),           /* TBD */

        /* TMSFAX-related statuses -- may or may not apply to KHE */

    KVEERR_KSE_USERO                = KERR_ERR( KSE_MOD, 0x01 ),           /* Error in user output function */
    KVEERR_KSE_MODE                 = KERR_ERR( KSE_MOD, 0x02 ),           /* Bad Parameter: Mode */
    KVEERR_KSE_OUT_SIZE             = KERR_ERR( KSE_MOD, 0x03 ),           /* Bad Parameter: Output Buffer Size */
    KVEERR_KSE_IBUFFER              = KERR_ERR( KSE_MOD, 0x04 ),           /* Bad Parameter: Input Buffer */
    KVEERR_KSE_OBUFFER              = KERR_ERR( KSE_MOD, 0x05 ),           /* Bad Parameter: Output Buffer */
    KVEERR_KSE_MINIMUM_BITS         = KERR_ERR( KSE_MOD, 0x06 ),           /* Bad Parameter: Minimum Bits */
    KVEERR_KSE_KFACTOR              = KERR_ERR( KSE_MOD, 0x07 ),           /* Bad Parameter: K-Factor */
    KVEERR_KSE_USERI                = KERR_ERR( KSE_MOD, 0x08 ),           /* Error in user input function */
    KVEERR_KSE_BP                   = KERR_ERR( KSE_MOD, 0x09 ),           /* Bad Parameter */
    KVEERR_KSE_INBITS               = KERR_ERR( KSE_MOD, 0x0A ),           /* Bad Parameter: Input Bits */
    KVEERR_KSE_OUTBITS              = KERR_ERR( KSE_MOD, 0x0B ),           /* Bad Parameter: Output Bits */
    KVEERR_KSE_BITOFF               = KERR_ERR( KSE_MOD, 0x0C ),           /* Bad Parameter: Bit Offset */
    KVEERR_KSE_DIR                  = KERR_ERR( KSE_MOD, 0x0D ),           /* Bad Parameter: Direction */
    KVEERR_KSE_DECOMP               = KERR_ERR( KSE_MOD, 0x0E ),           /* Decompression Error */
    KVEERR_KSE_DMODE                = KERR_ERR( KSE_MOD, 0x0F ),           /* Bad Parameter: Mode */
    KVEERR_KSE_DIBUFFER             = KERR_ERR( KSE_MOD, 0x10 ),           /* Bad Parameter: Input Buffer */
    KVEERR_KSE_OPERATION            = KERR_ERR( KSE_MOD, 0x11 ),           /* Bad Parameter: Operation */
    KVEERR_KSE_POSITION             = KERR_ERR( KSE_MOD, 0x12 ),           /* Bad Parameter: Position */
    KVEERR_KSE_TYPE                 = KERR_ERR( KSE_MOD, 0x13 ),           /* Bad Parameter: Type */
    KVEERR_KSE_INLINES              = KERR_ERR( KSE_MOD, 0x14 ),           /* Bad Parameter: Input Lines */
    KVEERR_KSE_OUTLINES             = KERR_ERR( KSE_MOD, 0x15 ),           /* Bad Parameter: Output Lines */
    KVEERR_KSE_WINRES               = KERR_ERR( KSE_MOD, 0x16 ),           /* Bad Parameter: Window Resolution */
    KVEERR_KSE_IMGRES               = KERR_ERR( KSE_MOD, 0x17 ),           /* Bad Parameter: Image Resolution */
    KVEERR_KSE_SCALE                = KERR_ERR( KSE_MOD, 0x18 ),           /* Bad Parameter: Scale */
    KVEERR_KSE_STRUCTURE            = KERR_ERR( KSE_MOD, 0x19 ),           /* Bad Parameter: Structure Pointer */
    KVEERR_KSE_INTERNAL1            = KERR_ERR( KSE_MOD, 0x1A ),           /* Internal Error #1 */
    KVEERR_KSE_TNOEXST              = KERR_ERR( KSE_MOD, 0x1B ),           /* Bad Parameter: Tile Number */
    KVEERR_KSE_TSCALE               = KERR_ERR( KSE_MOD, 0x1C ),           /* Scale Value For Tile Is Out Of Range */
    KVEERR_KSE_TMINRES              = KERR_ERR( KSE_MOD, 0x1D ),           /* Bad Parameter: Minimum Resolution */
    KVEERR_KSE_TNULLWIN             = KERR_ERR( KSE_MOD, 0x1E ),           /* Bad Parameter: Window Coordinate Pointer */
    KVEERR_KSE_TNULLBUF             = KERR_ERR( KSE_MOD, 0x1F ),           /* Bad Parameter: Tile Scratch Buffer Pointer */
    KVEERR_KSE_TNULLTIL             = KERR_ERR( KSE_MOD, 0x20 ),           /* Bad Parameter: Tile Pointer */

    KVEERR_KSE_INVALID_WINDOW       = KERR_ERR( KSE_MOD, 0x21 ),           /* TBD */
    KVEERR_KSE_INVALID_MODE         = KERR_ERR( KSE_MOD, 0x22 ),           /* TBD */
    KVEERR_KSE_NOT_WINDOWABLE       = KERR_ERR( KSE_MOD, 0x23 ),           /* TBD */
    KVEERR_KSE_INVALID_UNITS        = KERR_ERR( KSE_MOD, 0x24 ),           /* TBD */
    KVEERR_KSE_OUT_OF_WINDOWS       = KERR_ERR( KSE_MOD, 0x25 ),           /* TBD */
    KVEERR_KSE_INVALID_WINDOW_PARAM = KERR_ERR( KSE_MOD, 0x26 ),           /* TBD */
    KVEERR_KSE_INVALID_ARG          = KERR_ERR( KSE_MOD, 0x27 ),           /* TBD */
    KVEERR_KSE_OLD_VERSION          = KERR_ERR( KSE_MOD, 0x28 ),           /* Old version of software image engine */
    KVEERR_KSE_DIVIDE_ZERO          = KERR_ERR( KSE_MOD, 0x29 ),           /* Divide by zero */
    KVEERR_KSE_NO_KERNEL            = KERR_ERR( KSE_MOD, 0x2A ),           /* TBD */

        /***** KIE errors (mostly related to DLL loading */

    KVEERR_DLL_FILE_NOT_FOUND       = KERR_ERR( KIE_MOD, 0x01 ),           /* DLL not found */
    KVEERR_DLL_INSUFFICIENT_MEM     = KERR_ERR( KIE_MOD, 0x02 ),           /* Insufficient memory to load DLL */
    KVEERR_DLL_INVALID_FMT          = KERR_ERR( KIE_MOD, 0x03 ),           /* DLL file format is bad */
    KVEERR_DLL_UNKNOWN_ERR          = KERR_ERR( KIE_MOD, 0x04 ),           /* Unknown error encountered while loading DLL */

    KVEERR_KIPP_ERROR               = KERR_ERR( KVE_MOD, 0xFF ),           /* TBD */

   /* Status definitions that can be returned from user-supplied I/O functions. */

    KVEERR_USR_END_OF_STRIP         = KERR_ERR( KVE_USR_MOD, 0x01 ),       /* End of strip */
    KVEERR_USR_END_OF_IMAGE         = KERR_ERR( KVE_USR_MOD, 0x02 ),       /* End of image */
    KVEERR_USR_CHANGE_TO_RAW_STRIP  = KERR_ERR( KVE_USR_MOD, 0x03 ),       /* TBD */
    KVEERR_USR_CHANGE_TO_CMP_STRIP  = KERR_ERR( KVE_USR_MOD, 0x04 ),       /* TBD */


/* ========================= */
/* =====  FROM KERR.H  ===== */
/* ========================= */

    KERR_UNKNOWN_ERR                = KERR_ERR( KERR_MOD, 0x01 ),          /* Unknown error */
    KERR_TRUNCATED                  = KERR_ERR( KERR_MOD, 0x02 ),          /* Truncated */
    KERR_BAD_BUF_SIZE               = KERR_ERR( KERR_MOD, 0x03 ),          /* Bad buffer size */
    KERR_DLL_CANT_GET_PROC          = KERR_ERR( KERR_MOD, 0x04 ),          /* Unable to find required function in DLL */
    KERR_DLL_NOT_PREV_LOADED        = KERR_ERR( KERR_MOD, 0x05 ),          /* DLL was not previously loaded */
    KERR_DLL_FILE_NOT_FOUND         = KERR_ERR( KERR_MOD, 0x06 ),          /* Cannot load DLL -- file not found */
    KERR_DLL_INSUFFICIENT_MEM       = KERR_ERR( KERR_MOD, 0x07 ),          /* Cannot load DLL -- insufficient memory */
    KERR_DLL_INVALID_FMT            = KERR_ERR( KERR_MOD, 0x08 ),          /* Cannot load DLL -- invalid format */
    KERR_DLL_UNKNOWN_ERR            = KERR_ERR( KERR_MOD, 0x09 ),          /* Cannot load DLL -- reason unknown */
    KERR_DLL_NEED_NEWER_WIN         = KERR_ERR( KERR_MOD, 0x0A ),          /* Cannot load DLL -- requires more recent version of Windows */
    KERR_NO_CHUCKER                 = KERR_ERR( KERR_MOD, 0x0B ),          /* Driver not found */
    KERR_CHUCKER_OUT_OF_DATE        = KERR_ERR( KERR_MOD, 0x0C ),          /* Driver is obsolete */
    KERR_NO_SELECTORS_AVAIL         = KERR_ERR( KERR_MOD, 0x0D ),          /* No selectors available */
    KERR_NO_CHUCKER_RESPONSE        = KERR_ERR( KERR_MOD, 0x0E ),          /* Driver detected but will not respond */
    KERR_SCN_NOP                    = KERR_ERR( KIPP_MOD, 0x01 ),          /* No work done. */
    KERR_SCN_STARTED                = KERR_ERR( KIPP_MOD, 0x02 ),          /* Scanner started. */
    KERR_SCN_READ_DONE              = KERR_ERR( KIPP_MOD, 0x03 ),          /* Read from scanner completed. */
    KERR_SCN_WRITING_DATA           = KERR_ERR( KIPP_MOD, 0x04 ),          /* Writing data to disk. */
    KERR_SCN_WRITE_DATA_DONE        = KERR_ERR( KIPP_MOD, 0x05 ),          /* Write to disk completed. */ 
    KERR_SCN_DSP_DATA_XFER_DONE     = KERR_ERR( KIPP_MOD, 0x06 ),          /* Transfer of data to display completed. */
    KERR_SCN_OP_DONE                = KERR_ERR( KIPP_MOD, 0x07 ),          /* Scan operation completed. */
    KERR_SCN_OPEN_NOT_ISSUED        = KERR_ERR( KIPP_MOD, 0x08 ),          /* ScanStart attempted before ScanOpen issued. */
    KERR_SCN_START_NOT_ISSUED       = KERR_ERR( KIPP_MOD, 0x09 ),          /* ScanRun attempted before ScanStart issued. */
    KERR_SCN_JOB_SEPARATOR_SHEET    = KERR_ERR( KIPP_MOD, 0x10 ),          /* Job separator sheet detected. */
    KERR_SCN_PREVIEW_SCALE          = KERR_ERR( KIPP_MOD, 0x11 ),          /* Calculated the display scaling factors. */
        KERR_SCN_PAPER_IN_ADF           = KERR_ERR( KIPP_MOD, 0x12 ),          /* Paper detected in ADF.    */
    KERR_PRN_NOP                    = KERR_ERR( KIPP_MOD, 0x20 ),          /* No work done. */
    KERR_PRN_STARTED                = KERR_ERR( KIPP_MOD, 0x21 ),          /* Printer started. */
    KERR_PRN_READ_DONE              = KERR_ERR( KIPP_MOD, 0x22 ),          /* Read from printer completed. */
    KERR_PRN_OPERATION_DONE         = KERR_ERR( KIPP_MOD, 0x23 ),          /* Print operation completed. */
    KERR_PRN_OPEN_NOT_ISSUED        = KERR_ERR( KIPP_MOD, 0x24 ),          /* PrintStart attempted before PrintOpen issued. */
    KERR_PRN_START_NOT_ISSUED       = KERR_ERR( KIPP_MOD, 0x25 ),          /* PrintRun attempted before PrintStart issued. */
    KERR_PRN_ENGINE_NOTRESERVED     = KERR_ERR( KIPP_MOD, 0x26 ),          /* Print Engine not reserved. */
    KERR_PRN_SERIALIZE              = KERR_ERR( KIPP_MOD, 0x27 ),          /* Must serialize print operations. */
    KERR_PRN_QUEUE_EMPTY            = KERR_ERR( KIPP_MOD, 0x28 ),          /* Print Queue Empty. */
    KERR_PRN_QUEUE_FULL             = KERR_ERR( KIPP_MOD, 0x29 ),          /* Print Queue Full. */
        KERR_PSPEC_NOTINITIALIZED       = KERR_ERR( KIPP_MOD, 0x30 ),          /* Print Spec not initialized.    */
    
/* ========================= */
/* =====  FROM KOS.H   ===== */
/* ========================= */

    KOS_ERR_CRITSECT_FAIL           = KERR_ERR( KOS_MOD, 0x01 ),           /* Critical section failure */
    KOS_ERR_SEM_FAILURE             = KERR_ERR( KOS_MOD, 0x02 ),           /* Semaphore failure */
    KOS_ERR_SEM_NOT_OWNED           = KERR_ERR( KOS_MOD, 0x03 ),           /* Semaphore not owned */
    KOS_ERR_SEM_TIMEOUT             = KERR_ERR( KOS_MOD, 0x04 ),           /* Semaphore timeout */
    KOS_ERR_EXITLIST_FAIL           = KERR_ERR( KOS_MOD, 0x05 ),           /* TBD */
    KOS_ERR_PROCESS_FAIL            = KERR_ERR( KOS_MOD, 0x06 ),           /* TBD */
    KOS_ERR_TMPFILE_CREATE_FAIL     = KERR_ERR( KOS_MOD, 0x07 ),           /* Unable to create temporary file */


/* ========================= */
/* =====  FROM KUT.H   ===== */
/* ========================= */

    KUT_ERR_MULTIPLY_OVERFLOW       = KERR_ERR( KUT_MOD, 0x01 ),           /* Multiply overflow */
    KUT_ERR_GLOBAL_LOCK_FAILED      = KERR_ERR( KUT_MOD, 0x02 ),           /* Global lock failure */
    KUT_ERR_NO_GLOBAL_MEMORY        = KERR_ERR( KUT_MOD, 0x03 ),           /* No global memory */
    KUT_ERR_DIVIDE_BY_ZERO          = KERR_ERR( KUT_MOD, 0x04 ),           /* Divide by zero */
    KUT_ERR_READ_ONLY_PARAM         = KERR_ERR( KUT_MOD, 0x05 ),           /* Parameter is read-only */
    KUT_ERR_BAD_PARAM_TYPE          = KERR_ERR( KUT_MOD, 0x06 ),           /* Bad parameter type */
    KUT_ERR_BAD_PARAM_SIZE          = KERR_ERR( KUT_MOD, 0x07 ),           /* Bad parameter size */
    KUT_ERR_NOT_IMPLEMENTED         = KERR_ERR( KUT_MOD, 0x08 ),           /* Not implemented */
    KUT_ERR_BAD_CXT_BUF_DESC        = KERR_ERR( KUT_MOD, 0x09 ),           /* Bad context buffer descriptor */
    KUT_ERR_BAD_CXT_SIZE            = KERR_ERR( KUT_MOD, 0x0A ),           /* Bad context size */
    KUT_ERR_NULL_POINTER            = KERR_ERR( KUT_MOD, 0x0B ),           /* Null pointer */
    KUT_ERR_BAD_ROTATION            = KERR_ERR( KUT_MOD, 0x0C ),           /* Invalid rotation */
    KUT_ERR_UNSUPPORTED_HANDLE      = KERR_ERR( KUT_MOD, 0x0D ),           /* Unsupported memory handle type */
    KUT_ERR_BAD_FUNC_PARAM          = KERR_ERR( KUT_MOD, 0x0E ),           /* Bad function parameter */
    KUT_ERR_BAD_MAP_HANDLE          = KERR_ERR( KUT_MOD, 0x0F ),           /* Bad mapping handle */
    KUT_ERR_BAD_PARAM               = KERR_ERR( KUT_MOD, 0x10 ),           /* Bad parameter */
    KUT_ERR_ADDITION_OVERFLOW       = KERR_ERR( KUT_MOD, 0x11 ),           /* Addition overflow */
    KUT_ERR_SUBTRACTION_OVERFLOW    = KERR_ERR( KUT_MOD, 0x12 ),           /* Subtraction overflow */
    KUT_ERR_UNSUPPORTED_ROTATION    = KERR_ERR( KUT_MOD, 0x13 ),           /* Unsupported rotation */
    KUT_ERR_UNSUPPORTED_BLT_OP      = KERR_ERR( KUT_MOD, 0x14 ),           /* Unsupported BLT operation */
        KUT_ERR_ALARM_NOT_SET                   = KERR_ERR( KUT_MOD, 0x15 ),           /* Alarm never set */
        KUT_ERR_ALARM_NOT_RUNNING               = KERR_ERR( KUT_MOD, 0x16 ),           /* Alarm never started */
        KUT_ERR_ALARM_TIMEOUT                   = KERR_ERR( KUT_MOD, 0x17 ),           /* Alarm has timed out */


} KERR_ERR_ENUM, FAR *LP_KERR_ERR_ENUM;

/*****************************************************************************
* Function:     KERRGetStatusText
* Purpose:      Get the text string associated with a particular error/status code.
* Returns:      TBD
*****************************************************************************/
/* KERR_ERR_ENUM FAR PASCAL KERRGetStatusText(
/*      int             nStatus,        /* (I) The status in question */
/*      LPSTR   lpMsgBuf,       /* (O) Buffer into which msg string should be written */
/*      int             nBufSize        /* (I) The size of lpMsgBuf */
/* ); */
#endif /* kerr_h */
