/*****************************************************************************
* (c) Copyright Kofax Image Products, 1989.  All rights reserved.            *
*     Unauthorized use, duplication, or distribution is strictly prohibited. *
******************************************************************************
*
* File:                 kipp.h
* Purpose:              Kofax Image Processing Platform Header File
* Note:                 Should be used by including kippd.h, kippo.h or kippw.h 
*                               depending on the desired platform.
*
* $Revision:   1.47  $
*     $Date:   04 Dec 1992 21:17:40  $
* $NoAuthor:$
*
*    $NoLog:$
*****************************************************************************/
/*tab=4*/

#if !defined(kipp_h)
#define kipp_h

typedef HANDLE HIMDESC;                 /* Handle to Image Descriptor */
typedef HANDLE HPRTSPEC;                /* Handle to Print Spec */
typedef HANDLE HSCNSPEC;                /* Handle to Scan Spec */

#if !defined(DOS)
typedef DWORD (FAR PASCAL *FLTPROC)(    
  void FAR *lpCxt,
  WORD wOp,
  WORD wParam,
  DWORD dwParam,
  LPINT lpStat);
#endif


#define KIPPAPPNAME     "KIPP"          /* Kofax Image Processing Platform */

/***** Declarations required for TIFF and Sample Storage Filters respectively */

#if defined(DOS) || defined(UNIX)
    FLTPROC GetTiffStorageProc(void);       /* for TIFF filter */
#ifdef DOS
    FLTPROC GetEmsStorageProc(void);        /* for EMS filter */
#else
    FLTPROC GetMemStorageProc(void);        /* for memory storage filter */
#endif
    FLTPROC GetImgStorageProc(void);        /* for IMG Sample filter */
    FLTPROC GetPCXStorageProc(void);        /* for PCX Sample filter */
#endif

/***** Storage Process Operations -- operations performed by StorageProc() */

#define KSP_INIT                  1     /* initialize image descriptor */
#define KSP_nGETEXTSIZE           2     /* get imdesc extension size */
#define KSP_DELETE                3     /* delete imdesc (free resources) */
#define KSP_lGETATTR              4     /* get image attribute */
#define KSP_lSETATTR              5     /* set image attribute */
#define KSP_lTESTATTRBIT          6     /* test bit of image attribute */
#define KSP_lCLRATTRBIT           7     /* clear bit of image attribute */
#define KSP_lSETATTRBIT           8     /* set bit of image attribute */
#define KSP_OPENIMAGE            10     /* open image */
#define KSP_CLOSEIMAGE           11     /* close image */
#define KSP_CREATEIMAGE          12     /* create and open image */
#define KSP_REMOVEIMAGE          13     /* remove image */
#define KSP_SEEKIMAGE            14     /* set to logical position of image */
#define KSP_READIMAGE            15     /* get next band/strip of image data */
#define KSP_WRITEIMAGE           16     /* put next band/strip of image data */
#define KSP_CREATECOMPATIBLE     17     /* create compatible image descriptor */



/*
    IMAGE ATTRIBUTES -- image attributes for KSP_GETATTR and KSP_SETATTR

        NOTES: (1) some attributes such as KIA_bMODFIED are part of
                   other attributes (in this case dwSTATUS)
               (2) dwEXPERROFFSET and wEXPERRLINE are valid after a compressed
                   data error has been returned from a expansion operation 
                   such as DisplayImage(), PrintImage(), or ReadImage()...
                   One attribute returns the offset into the compresed codes,
                   the other the line in which the error occurred.  As of this
                   date, these attributes are NOT fully supported in all cases,
                   nor are they guaranteed to be supported in the future;
                   they are for internal use only.
*/
                                        /** COMMON IMAGE ATTRIBUTES **/
#define KIA_hTASK                 1     /* owner of the image desc */
#define KIA_dwSTATUS              2     /* image status flags */
#define KIA_bMODIFIED             4     /* set/clr image modified status flg */
#define KIA_wDEVICEERRORCODE      7     /* last device specific error */
#define KIA_wOSERRORCODE          8     /* last OS error code if available */
#define KIA_wWIDTH               10     /* width in pixels (X:horizontal) */
#define KIA_wLENGTH              11     /* length in pixels (Y:vertical) */
#define KIA_wTILEWIDTH           12     /* tile width (X:horizontal) */
#define KIA_wTILELENGTH          13     /* tile length (Y:vertical) */
#define KIA_wPITCH               14     /* bytes per scan line */
#define KIA_wHDPI                15     /* horizontal DPI */
#define KIA_wVDPI                16     /* vertical DPI */
#define KIA_wHPOSITION           17     /* offset from left side of page */
#define KIA_wVPOSITION           18     /* offset from top of page */
#define KIA_dwFORMAT             20     /* image data format flags */
#define KIA_byTIMEFILL           21     /* compression time fill value */
#define KIA_byKPARAM             22     /* compression K-parameter value */
#define KIA_wSAMPLESPERPIXEL     25     /* number of samples per pixel */
#define KIA_wBITSPERSAMPLE       26     /* number of bits per sample */
#define KIA_dwPAPERSIZE          27     /* paper size code */
#define KIA_dwEXPERROFFSET       28     /* {reserved} see NOTE (2) */
#define KIA_wEXPERRLINE          29     /* {reserved} see NOTE (2) */

                                        /** COMMON STORAGE ATTRIBUTES **/
#define KIA_lpPATHNAME          128     /* pathname string */
#define KIA_nPATHNAMELENGTH     129     /* length of pathname string */
#define KIA_nFILEACCESSMODES    130     /* used when opening image files */
#define KIA_nFILEPERMMODES      131     /* used when creating image files */
#define KIA_nRESOLUTIONUNIT     132     /* 2=inches, 3=centimeters */
#define KIA_dwDATASIZE          133     /* size of image data in bytes */
#define KIA_bPAGED              134     /* whole lines in EMS pages */
#define KIA_wPAGE1ST            135     /* 1st logical EMS pg for image data */
#define KIA_wEMSHANDLE          136     /* EMS handle to image pages */
#define KIA_wEMSSEGMENT         137     /* EMS segment adress */
#define KIA_nEXTRAHANDLES       138     /* EMS number of handles obtained */
#define KIA_nPGSPERHANDLE       139     /* EMS pages to allocate per handle */
#define KIA_dwSTARTDATA         140     /* EMS image data offset in 1st page */

#define KIA_wXIO                141     /* current horiz storage IO position */
#define KIA_wYIO                142     /* current vert storage IO position */
                                        /* users may request that defines
                                           be added to the common storage
                                           attributes section by contacting
                                           Kofax Image Products */

                                        /** USER DEFINED ATTRIBUTES **/
#define KIA_tagUSERATTRIBUTES   256     /* start of user defined attributtes */

/*
    Image Status Flags (KIA_dwSTATUS)

    low  word = ongoing status
    high word = reset before each operation
*/
#define KIS_OPEN            0x000001L   /* image is open */
#define KIS_MODIFIED        0x000002L   /* image has been modified */
#define KIS_READONLY        0x000004L   /* image is read only */
                                        /**/
#define KIS_LENGTHADJUSTED  0x010000L   /* length fld changed to actual len */

/*
    Image Data Format Flags Definitons (for KIA_dwFORMAT):
        The low order eight bits of this DWORD field contain the compression
        scheme.  Other bits in this field describe any additional information
        about the image data format.
*/
#define KIF_CMPMASK         0x0000FFL   /* compression type mask */
#define KIF_NONE            0x000000L   /* no compression */
#define KIF_USER            KIF_NONE    /* user compression */
#define KIF_PACKBITS        0x000001L   /* PackBits compression */
#define KIF_PACKBYTES       0x000002L   /* PackBytes compression */
#define KIF_TIFF2NOEOL      0x000004L   /* TIFF Data compression scheme 2 */
#define KIF_GROUP3          0x000010L   /* CCITT group 3 compression */
#define KIF_GROUP4          0x000120L   /* CCITT group 4/2D compression */
                                        /**/
#define KIF_2DIMENSIONAL    0x000100L   /* two dimensional compression */
#define KIF_NOEOL           0x000200L   /* no end of line codes */
#define KIF_ALIGNEDEOL      0x000400L   /* EOL's end on byte boundaries */
#define KIF_GROUP32D        0x000110L   /* CCITT group 3/2D compression */
#define KIF_GROUP42D        0x000120L   /* CCITT group 4/2D compression */
                                        /**/
#define KIF_PLANES          0x000800L   /* samples stored in separate planes */
#define KIF_COLOR           0x001000L   /* color inverted: 0=black, 1=white */
#define KIF_BITSEX          0x002000L   /* bit order inverted: high to low */
#define KIF_COLORCODES      0x004000L   /* compressed codes are clr inverted */
#define KIF_BITSEXCODES     0x008000L   /* compressed codes are bit sexed */
#define KIF_WORDALIGN       0x010000L   /* word aligned line (2bytes) */
#define KIF_DWORDALIGN      0x020000L   /* double word aligned line (4bytes) */
                                        /**/
#define KIF_LINE            0x100000L   /* scanned in line art mode */
#define KIF_DITHERED        0x200000L   /* scanned in dithered mode */
#define KIF_PHOTO           0x400000L   /* scanned in photo mode */

/*
    Image Data Format defaults:
        ~KIF_COLOR:     color:      1=black, 0=white (WhiteIsZero)
        ~KIF_BITSEX:    bitsex:     samples stored from low to high order bit
        ~KIF_*ALIGN:    alignment:  byte aligned
        ~KIF_PLANES:    config:     samples stored contiguously, single image plane
*/
/*
    Storage Filter Image File Access and Permission modes
        (for KIA_nFILEACCESSMODES and KIA_nFILEPERMMODES)
*/
#define KIF_READONLY        0x0000
#define KIF_WRITEONLY       0x0001
#define KIF_READWRITE       0x0002

/*
    Storage Filter Image Resolution Unit of Measurement
        (for KIA_nRESOLUTIONUNIT)
*/
#define KIR_INCH            2
#define KIR_CENTIMETER      3

/*
    Storage filter Image I/O flags: (for KIA_READIMAGE and KIA_WRITEIMAGE)
        Passed via *lpStat from or to KIA_READIMAGE and KIA_WRITEIMAGE
        storage filter operations.  KIO_ENDOFIMAGE implies KIO_ENDOFTILE.
*/
#define KIO_ENDOFIMAGE      0x0001          /* END OF IMAGE FLAG */
#define KIO_ENDOFTILE       0x0002          /* END OF TILE FLAG */
#define KIO_STARTRAWTILE    0x0004          /* START OF RAW TILE */




/*
    All dwOpt options -- 
        NOT ALL OPTIONS SUPPORTED BY EVERY OPERATION.
        Use KDI_* for DisplayImage(), KRE_* for ReadImage(), KSC_* for
        ScanImage(), etc...
*/
#define KOP_NONE            0x000000L       /* no options */

#define KOP_COLOR           0x000010L       /* invert color */
#define KOP_BITSEX          0x000020L       /* invert bit sex */
#define KOP_FILLWINDOW      0x000040L       /* auto scale to fill window */
#define KOP_FASTSCALE       0x000080L       /* favor faster scaling methods */
#define KOP_FITINWINDOW     0x000100L       /* auto scale to fit in window */
#define KOP_PREVIEWSCALE    0x000200L       /* preview actual scale factors */

#define KOP_HSATURATION     0x000400L       /* enable horizontal saturation */
#define KOP_VSATURATION     0x000800L       /* enable vertical saturation */
#define KOP_SATURATION      (KOP_HSATURATION|KOP_VSATURATION)

#define KOP_ROT0            0x000000L       /* no rotation (default) */
#define KOP_ROT90           0x001000L       /* rotate 90x */
#define KOP_ROT180          0x002000L       /* rotate 180x */
#define KOP_ROT270          0x003000L       /* rotate 270x */
#define KOP_ROTMASK         0x003000L       /* rotation flag mask */
#define KOP_ENDOFTILE       0x004000L       /* end of tile (or strip) */
#define KOP_ENDOFIMAGE      0x008000L       /* end of image */

#define KOP_GREYDISPLAY     0x010000L       /* enable grey display (WINDOWS only) */

#define KOP_BEGIN           0x100000L       /* begin a new operation */
#define KOP_CONTINUE        0x200000L       /* continue with an operation */
#define KOP_END             0x400000L       /* end an operation */
#define KOP_ABORT           0x800000L       /* abort an operation */

/*
    dwOpt for DisplayImage() 
*/
#define KDI_NONE            KOP_NONE
#define KDI_COLOR           KOP_COLOR
#define KDI_BITSEX          KOP_BITSEX
#define KDI_FASTSCALE       KOP_FASTSCALE
#define KDI_FILLWINDOW      KOP_FILLWINDOW
#define KDI_FITINWINDOW     KOP_FITINWINDOW
#define KDI_PREVIEWSCALE    KOP_PREVIEWSCALE
#define KDI_HSATURATION     KOP_HSATURATION
#define KDI_VSATURATION     KOP_VSATURATION
#define KDI_SATURATION      (KDI_HSATURATION|KDI_VSATURATION)
#define KDI_ROT0            KOP_ROT0
#define KDI_ROT90           KOP_ROT90
#define KDI_ROT180          KOP_ROT180
#define KDI_ROT270          KOP_ROT270
#define KDI_ROTMASK         KOP_ROTMASK
#define KDI_GREYDISPLAY     KOP_GREYDISPLAY

/*
    dwOpt for PrintImage()
*/
#define KPR_NONE            KOP_NONE
#define KPR_ROT0            KOP_ROT0
#define KPR_ROT90           KOP_ROT90
#define KPR_ROT180          KOP_ROT180
#define KPR_ROT270          KOP_ROT270
#define KPR_ROTMASK         KOP_ROTMASK
#define KPR_HSATURATION     KOP_HSATURATION
#define KPR_VSATURATION     KOP_VSATURATION
#define KPR_PREVIEWSCALE    KOP_PREVIEWSCALE
#define KPR_FITINWINDOW     KOP_FITINWINDOW
#define KPR_FILLWINDOW      KOP_FILLWINDOW
#define KPR_FASTSCALE       KOP_FASTSCALE

/*
    dwOpt for ScanImage()
*/
#define KSC_NONE            KOP_NONE

/*
    dwOpt for ReadImage()
*/
#define KRE_NONE            KOP_NONE
#define KRE_COLOR           KOP_COLOR
#define KRE_BITSEX          KOP_BITSEX
#define KRE_PREVIEWSCALE    KOP_PREVIEWSCALE
#define KRE_BEGIN           KOP_BEGIN
#define KRE_CONTINUE        KOP_CONTINUE
#define KRE_END             KOP_END
#define KRE_ABORT           KOP_ABORT
#define KRE_ROT0            KOP_ROT0
#define KRE_ROT90           KOP_ROT90
#define KRE_ROT180          KOP_ROT180
#define KRE_ROT270          KOP_ROT270
#define KRE_ROTMASK         KOP_ROTMASK
#define KRE_HSATURATION     KOP_HSATURATION
#define KRE_VSATURATION     KOP_VSATURATION

/*
    dwOpt for WriteImage()
*/
#define KWR_NONE            KOP_NONE
#define KWR_COLOR           KOP_COLOR
#define KWR_BITSEX          KOP_BITSEX
#define KWR_ENDOFTILE       KOP_ENDOFTILE
#define KWR_ENDOFIMAGE      KOP_ENDOFIMAGE
#define KWR_BEGIN           KOP_BEGIN
#define KWR_CONTINUE        KOP_CONTINUE
#define KWR_END             KOP_END
#define KWR_ABORT           KOP_ABORT

/*
    dwOpt for CopyImage() 
*/
#define KCP_FITINDESTINATION KOP_FITINWINDOW
#define KCP_NOZOOMOUT        0x010000L
#define KCP_ROT0             KOP_ROT0
#define KCP_ROT90            KOP_ROT90
#define KCP_ROT180           KOP_ROT180
#define KCP_ROT270           KOP_ROT270

/*
    dwOpt for ScanDisplayImage()
*/
/*** USE OPTIONS FOR ScanImage() and DisplayImage() ***/

    
    
/* 
    Image x,y position definition
*/
typedef union tagXYPOS
{   struct
    {
#ifdef HIGH_ENDIAN
        COORD x;
        COORD y;
#else
        COORD y;
        COORD x;
#endif
    } xy;
    DWORD pt;
} XYPOS, *PXYPOS, NEAR *NPXYPOS, FAR *LPXYPOS;


/*
    Commonly Used Scaling Factors (N:M) {where N is in MSB M is in LSB}
*/
#define KSC_1TO1            0x0101
#define KSC_3TO2            0x0302
#define KSC_4TO3            0x0403
#define KSC_2TO1            0x0201
#define KSC_3TO1            0x0301
#define KSC_4TO1            0x0401
#define KSC_6TO1            0x0601
#define KSC_8TO1            0x0801
#define KSC_12TO1           0x0C01
#define KSC_16TO1           0x1001




/*
    Print Parameters

*/

#define KPP_CAPSID(x) ((WORD)x | 0x8000)    
#define KPP_PARAMID(x) ((WORD)x & 0x7FFF)   

#define KPP_wDPI            700
/* Use printer caps dpi values */

#define KPP_dwPAPERSIZE     702 
/* Use printer caps paper sizes */
 #define USER_PAPERSIZE      0x40000000L

#define KPP_wPAPERSOURCE    703
/* Paper Sources */
 #define ENVELOPE            0x0001
 #define TRAY1               0x0002
 #define TRAY2               0x0004
 #define TRAY3               0x0008
 #define AUTO_TRAY           0x0040
 #define HOLDING_TRAY        0x0010
 #define MANUAL_FEED         0x0020       /* Not supported */

#define KPP_wPAPERDEST      704
/* Paper destinations */
 #define PANEL_PAPERDEST     0x0001
 #define STACKER1            0x0002
 #define STACKER2            0x0004
 #define STACKER3            0x0008
 #define HOLDING_TRAY        0x0010

#define KPP_fFLIPCOLOR      706
#define KPP_fFLIPBITSEX     707

#define KPP_dwSTATUS        712
/* Use Printer statuses for paper jam and no paper */
  #define PRINT_FAIL        0x80000000L         /* general failure */
  #define NO_CART           0x40000000L         /* no cartridge */
  #define COVER_OPEN        0x04000000L         /* top is popped */
  #define TEST_PRINT        0x02000000L         /* test print in progress */
  #define BAD_FIXING        0x00400000L         /* fixing assembly malfunction */
  #define BAD_LIGHT         0x00200000L         /* bad laser diode (no BD) */
  #define MOTOR_MALFUNC     0x00100000L         /* scanning motor malfunction */
  #define TONER_LOW         0x00000400L         /* toner low */

#define KPP_wCOPIES         711
#define KPP_fSQUEEZE        713
#define KPP_lpSCALE         714
#define KPP_lpSRCRECT       715
#define KPP_lpDESTRECT      716

#define KPP_wPAPERUNITS     717
/* Use printer units */

#define KPP_wPRINTTIMEOUT   718
#define KPP_fDPISCALING     719

#define KPP_wDESTUNITS      720
/* Use printer units */

#define KPP_wPAPERWIDTH     721
#define KPP_wPAPERLENGTH    722
#define KPP_wCOPIESPRINTED  723

#define KPP_wSRCUNITS       724
#define KPP_dwTRAYPAPERSIZE 725
#define KPP_fTOPJUSTIFY     726
#define KPP_fLEFTJUSTIFY    727

#ifdef WIN
#define KPP_fNO_DEFAULT_IDLE_FUNC   730 /* Disable use of a default idle
                                           function. In Windows, the yield()
                                           function is the default idle 
                                           function. The default setting for
                                           this parameter is FALSE. */
#endif


#define KPC_wDPI            KPP_CAPSID(KPP_wDPI)
#define KPC_dwPAPERSIZE     KPP_CAPSID(KPP_dwPAPERSIZE)
#define KPC_wPAPERUNITS     KPP_CAPSID(KPP_wPAPERUNITS)
#define KPC_wPAPERSOURCE    KPP_CAPSID(KPP_wPAPERSOURCE)
#define KPC_wPAPERDEST      KPP_CAPSID(KPP_wPAPERDEST)
#define KPC_fSQUEEZE        KPP_CAPSID(KPP_fSQUEEZE)
#define KPC_wDESTUNITS      KPP_CAPSID(KPP_wDESTUNITS)
#define KPC_wPAPERLENGTH    KPP_CAPSID(KPP_wPAPERLENGTH)
#define KPC_wPAPERWIDTH     KPP_CAPSID(KPP_wPAPERWIDTH)
#define KPC_wSRCUNITS       KPP_CAPSID(KPP_wSRCUNITS)
#define KPC_fTOPJUSTIFY     KPP_CAPSID(KPP_fTOPJUSTIFY)
#define KPC_fLEFTJUSTIFY    KPP_CAPSID(KPP_fLEFTJUSTIFY)


/*
    Scan Parameters
*/
/*
 *  The definitions for KSP_wHDPI and KSP_wVDPI are only here on a
 *  temporary basis.  They will be removed in the future.
 */
#define KSP_wDPI                800
#define KSP_wHDPI               KSP_wDPI
#define KSP_wVDPI               KSP_wDPI
 #define PANEL_DPI               0x8000 
 #define AUTO_DPI                0x4000
 #define DPI_75                  0x0001
 #define DPI_100                 0x0002
 #define DPI_150                 0x0004
 #define DPI_200                 0x0008
 #define DPI_240                 0x0010
 #define DPI_300                 0x0020
 #define DPI_400                 0x0040
 #define DPI_600                 0x0080

#define KSP_dwPAPERSIZE         802
 #define AUTO_PAPERSIZE          0x40000000L
 #define PANEL_PAPERSIZE         0x80000000L
 #define UNKNOWN_PAPERSIZE       0x00000000L

 #define ASIZE_PAPER             0x00010000L
 #define LETTER_PAPER            0x00010000L
 #define LEGAL_PAPER             0x00020000L
 #define BSIZE_PAPER             0x00040000L
 #define CSIZE_PAPER             0x00080000L
 #define DSIZE_PAPER             0x00100000L
 #define ESIZE_PAPER             0x00200000L
 #define EXECUTIVE_PAPER         0x00400000L

 #define A0_PAPER                0x00000001L
 #define A1_PAPER                0x00000002L
 #define A2_PAPER                0x00000004L
 #define A3_PAPER                0x00000008L
 #define A4_PAPER                0x00000010L
 #define A5_PAPER                0x00000020L

 #define B0_PAPER                0x00000100L
 #define B1_PAPER                0x00000200L
 #define B2_PAPER                0x00000400L
 #define B3_PAPER                0x00000800L
 #define B4_PAPER                0x00001000L
 #define B5_PAPER                0x00002000L

#define KSP_byPAPERSOURCE       803
 #define AUTO_PAPERSOURCE        0x40
 #define PANEL_PAPERSOURCE       0x80
 #define SCANNER_MANUAL          0x01
 #define SCANNER_FLATBED         0x02
 #define SCANNER_ADF             0x04

#define KSP_bySTACKER           804
 #define AUTO_STACKER            0x40
 #define PANEL_STACKER           0x80
 #define STACKER_1               0x01
 #define STACKER_2               0x02
 #define STACKER_3               0x04
 #define UPPER_STACKER           1
 #define LOWER_STACKER           2


#define KSP_byORIENTATION       805
 #define AUTO_ORIENTATION        0x40
 #define PANEL_ORIENTATION       0x80
 #define PORTRAIT                0x01
 #define LANDSCAPE               0x02

#define KSP_byCOLOR             806
 #define PANEL_COLOR             0x80
 #define AUTO_COLOR              0x40
 #define CCITTCOLOR              0x01
 #define REVERSECOLOR            0x02

#define KSP_byBITSEX            807
 #define PANEL_BITSEX            0x80
 #define AUTO_BITSEX             0x40
 #define CCITTBITSEX             0x01
 #define REVERSEBITSEX           0x02

#define KSP_byDUPLEX            808
 #define PANEL_DUPLEX            0x80
 #define SINGLE_SIDE             0x01
 #define DUPLEX                  0x02

#define KSP_bySTARTMODE         809
 #define START_PAPERPRESENT       1
 #define START_IMMEDIATE          2
 #define START_PANEL              4

#define KSP_wINTENSITY          810
 #define AUTO_INTENSITY          0x4000
 #define PANEL_INTENSITY         0x8000
 #define LIGHTEST_INTENSITY      0x0001
 #define VERYLIGHT_INTENSITY     0x0002
 #define LIGHT_INTENSITY         0x0004
 #define NORMAL_INTENSITY        0x0008
 #define DARK_INTENSITY          0x0010
 #define VERYDARK_INTENSITY      0x0020
 #define DARKEST_INTENSITY       0x0040

#define KSP_wCONTRAST           811
 #define AUTO_CONTRAST           0x4000
 #define PANEL_CONTRAST          0x8000
 #define LOWEST_CONTRAST         0x0001
 #define VERYLOW_CONTRAST        0x0002
 #define LOW_CONTRAST            0x0004
 #define NORMAL_CONTRAST         0x0008
 #define HIGH_CONTRAST           0x0010
 #define VERYHIGH_CONTRAST       0x0020
 #define HIGHEST_CONTRAST        0x0040

#define KSP_bySCANMODE          812
 #define AUTO_SCANMODE           0x40
 #define PANEL_SCANMODE          0x80
 #define LINE_MODE               0x01
 #define PHOTO_MODE              0x02
 #define MIXED_MODE              0x04

/*
 *  These scan parameters are not fully implemented.  They can be used in
 *  calls to SetScanParam() and GetScanParam() but will have no bearing on
 *  the scanning process.
 */
#define KSP_wHOFFSET            813     /* NO LONGER SUPPORTED */
#define KSP_wVOFFSET            814     /* NO LONGER SUPPORTED */
#define KSP_wWIDTH              815     /* NO LONGER SUPPORTED */
#define KSP_wLENGTH             816     /* NO LONGER SUPPORTED */

#define KSP_dwSTATUS            817     /* READ-ONLY */
 #define NO_PAPER               0x01L
 #define PAPER_JAM              0x10L

/*
    Scaling to be applied to the resulting image.  This requires a long
    pointer to a scaling structure.
*/
#define KSP_lpSCALE             819     /* NOT SUPPORTED YET */

/*
    Picking Rectangle:  requires a long pointer to a rectangle.  The units
    of resolution can be set via the KSP_wUNITS attribute.  The origin
    of the coordinate system can be set via KSP_wORIGIN.  The default is
    no picking rectangle.  This can be selected by using a NULL pointer for
    the picking rectangle.  If the width of the picking rectangle is equal to
    zero then the resulting width is the default width minus the left edge
    of the picking rectangle.  If the length of the picking rectangle is 
    equal to zero then the resulting length is the actual length scanned
    minus the top edge of the picking rectangle.

    When no picking rectangle is specified the default width and length are
    computed from the paper size and DPI.  The default width is WORD aligned.
*/
#define KSP_lpPICKINGRECT       820

#define KSP_wUNITS              821
 #define PIXELS                 1
 #define INCHES                 2       /* NOT SUPPORTED YET */
 #define CENTIMETERS            4       /* NOT SUPPORTED YET */

#define KSP_wORIGIN             822
 #define SCANNED_ORIGIN         1
 #define PAPER_ORIGIN           2       /* NOT SUPPORTED YET */

/*
    Memory usage parameter.  EMS memory may be utilized (if present) to hold
    any scan data overflow that may occur when scanning images that produce
    a large amount of data.  This parameter may be used to disable the use of
    EMS memory during a scan.  The default usage mode enables EMS memory usage
    enabled if EMS memory is available.
*/
#define KSP_wMEMORYUSAGE        824
 #define ENABLE_EMS             0x01    /* enable EMS usage during scan */
 #define DISABLE_EMS            0x02    /* disable EMS usage during scan */


#define KSP_wSCANTIMEOUT        825     /* User settable scan timeout   */
                                        /* The dwValue is the scanner   */
                                        /* timeout value in seconds.    */

#define KSP_wDITHER             826
 #define PANEL_DITHER            0x8000
 #define NO_DITHER               0x0001
 #define TYPE0_DITHER            0x0002
 #define TYPE1_DITHER            0x0004
 #define TYPE2_DITHER            0x0008
 #define TYPE3_DITHER            0x0010

/*
    Width detection call back function address parameter (requires a
    LPFN_ADJUST_DETECTED_WIDTH (see below) parameter).  When width 
    detection is specified (a picking rectangle whose width is 0), 
    then the application program's call back function is called to adjust the 
    image width.  If no call back function is specified, the detected 
    width is used for the scan.
*/
#define KSP_lpfnADJUSTDETECTEDWIDTH 827 /* Auto width detection call back */
                                        /* function pointer.            */
#define KSP_fDRAMSCAN               828 /* More reliable scanning mode  */

#define KSP_fDETECTJOBSEPARATOR     829 /* Detect the job separator sheet */
                                        /* if the scanner is capable.    */
#ifdef WIN
#define KSP_fNO_DEFAULT_IDLE_FUNC   830 /* Disable use of a default idle
                                           function. In Windows, the yield()
                                           function is the default idle 
                                           function. The default setting for
                                           this parameter is FALSE. */
#endif

/*
    Scanner Capabilities
    (these Scan Parameters are read-only)
*/
#define KSC_wDPI                880
#define KSC_dwPAPERSIZE         881
#define KSC_byFEEDER            882
#define KSC_bySTACKER           883
#define KSC_bySCANMODE          884
#define KSC_byORIENTATION       885
#define KSC_byDUPLEX            886
#define KSC_byCOLOR             887
#define KSC_byBITSEX            888
#define KSC_fPICKING            889
#define KSC_wINTENSITY          890
#define KSC_wCONTRAST           891
#define KSC_bySTARTMODE         892
#define KSC_wUNITS              893
#define KSC_wORIGIN             894
#define KSC_fSCALING            895
#define KSC_wDITHER             896
#define KSC_fDRAMSCAN           897

/*
    RESOURCE INFORMATION ITEMS -- information types for GetResourceInfo()
*/                                      /** COMMON RESOURCE INFO ITEMS **/
#define KRI_wCLASS                1     /* classification of resource */
#define KRI_lpDESCRIPTION         2     /* description of classification */
#define KRI_wLOGICALUNIT          3     /* logical unit number */
#define KRI_lpLOGICALNAME         4     /* logical unit name */
#define KRI_wVERSION              5     /* version number (MAJOR/minor) */
#define KRI_lpVERSION             6     /* version number (ASCII string) */
#define KRI_lpFILENAME            7     /* associated DOS file name */

                                        /** COMMON HARDWARE INFO ITEMS **/
#define KRI_byHWINTERRUPT        32     /* hardware interrupt number */
#define KRI_wIOBASE              33     /* base I/O port number */
#define KRI_dwMEMORYSIZE         34     /* physical memory size */
#define KRI_byASSEMBLYVERSION    35     /* hardware assembly version */
#define KRI_wSECRETNUMBER        36     /* hardware secret number (PAL protection) */

                                        /** COMMON SOFTWARE INFO ITEMS **/
#define KRI_bySWVECTOR           64     /* software interrupt vector */
#define KRI_dwDRIVERSIZE         65     /* driver size in memory */

                                        /** CURRENTLY SPECIFIC TO KF-910 */
#define KRI_wREVISION            80     /* SW revision number */
#define KRI_lpSERIALNUMBER       81     /* SW serial number */

                                        /** COMMON PERIPHERAL INFO ITEMS **/
#define KRI_lpMAKE               96     /* manufacturer name */
#define KRI_lpMODEL              97     /* model number */

#define KRI_tagMAXITEM          128     /* end of info item list */

/*
    Resource Classifications (KRI_wCLASS)
*/
#define KRC_CHUCKER               1     /* KIPP Engine Driver */
#define KRC_9200                  2     /* KIPP 9200 Engine */
#define KRC_9100                  3     /* KIPP 9100 Engine */
#define KRC_910                   4     /* KIPP 910 Engine */
#define KRC_DIAG                  5     /* Diagnostic */
#define KRC_PRINTER               6     /* Document Printer */
#define KRC_PRINTAUX              7     /* Auxiliary Printer */
#define KRC_SCANNER               8     /* Document Scanner */
#define KRC_SCANAUX               9     /* Auxiliary Scanner */
#define KRC_9500                10    /* RAVEN 9500 */
#define KRC_9250                11    /* KIPP 9200 TURBO   */
#define KRC_9150                12    /* KIPP 9100 TURBO   */
#define KRC_AIC          13     /* KIPP engine controller */
#define KRC_7300         16     /* Scanner controller */
#define KRC_7400         17     /* Printer controller */
#define KRC_9210         18     /* Enhanced document processor */

/*
    Bit flags to control the action of the Printer and Scanner Communication
    functions.
*/
#define COM_SEND                0x01
#define COM_RECEIVE             0x10
#define COM_SENDRECEIVE         (COM_SEND | COM_RECEIVE)

/*
    KIPP ERRORS
*/                                      /** possible KIPP error values **/
#define KER_OK                    0     /* no error */
#define KER_FAIL                 -1     /* general failure */
#define KER_NA                   -2     /* feature not yet available */
#define KER_ACCESS               -3     /* access violation */
#define KER_CORRUPTED            -4     /* internal data structures corrupted */
#define KER_OPEN                 -5     /* image already opened */
#define KER_NOTOPEN              -6     /* image not opened */
#define KER_NOIMAGE              -7     /* can't open image */
#define KER_EXISTS               -8     /* image already exists */
#define KER_NOMEMORY             -9     /* no memory available */
#define KER_MEMORYFAIL          -10     /* failed to lock or free memory */
#define KER_READONLY            -11     /* read-only attribute */
#define KER_READFAIL            -12     /* file read error occurred */
#define KER_WRITEFAIL           -13     /* file write error occurred */
#define KER_BADHEADER           -14     /* bad image header found by filter */
#define KER_BADPOSITION         -15     /* bad image position requested */
#define KER_BADHANDLE           -16     /* invalid descriptor or spec handle */
#define KER_BADIMDESC           -17     /* image descriptor is corrupted */
#define KER_BADSPEC             -18     /* scan or print spec is corrupted */
#define KER_ENDOFIMAGE          -19     /* end of image detected */
#define KER_ENDOFTILE           -20     /* end of tile or strip detected */
#define KER_ENDOFBANDSRC        -21     /* user band source exhausted */
#define KER_CMPDATAERROR        -22     /* error in compressed data */
#define KER_NOHWND              -23     /* HWND parameter not supplied */
#define KER_BADPARAM            -24     /* bad parameter */
#define KER_PARAMID             -25     /* invalid parameter id specified */
#define KER_PARAMVALUE          -26     /* invalid pararmeter value specified */
#define KER_TRUNCATED           -27     /* user buffer too small */
#define KER_TIMEOUT             -28     /* timeout (engine, device, or KIPP) */
#define KER_NODRIVER            -29     /* engine driver not found */
#define KER_ENGINEFAIL          -30     /* engine failure */
#define KER_NOEMM               -31     /* no expanded memory manager */
#define KER_EMMFAIL             -32     /* expanded memory manager error */
#define KER_NOTCAPABLE          -33     /* device not capable */
#define KER_POWEROFF            -34     /* device power off */
#define KER_NOTREADY            -35     /* device not ready */
#define KER_TONERLOW            -36     /* device low on toner */
#define KER_NOPAPER             -37     /* device is out of paper */
#define KER_PAPERJAM            -38     /* device paper jam */
#define KER_DEVICESTOP          -39     /* stop button pushed on device */
#define KER_DEVICEFAIL          -40     /* unspecified device error */
#define KER_RESERVED            -41     /* device already reserved */
#define KER_NOTCONFIGURED       -42     /* device not configured */
#define KER_OVERRUN             -43     /* device data overrun */
#define KER_UNDERRUN            -44     /* device data underrun */
#define KER_BADDC               -45     /* display context is corrupted */
#define KER_PERIPHERALBUSY      -46     /* engine busy */
#define KER_DRIVERFAIL          -47     /* driver error */
#define KER_SEMFAIL             -48     /* semaphore failure */
#define KER_BADIMAGEWIDTH       -49     /* image width too small or too large */
#define KER_NOADAPTORMEMORY     -50     /* no adaptor memory available */
#define KER_ENGINEBUSY          -51     /* Requested engine is busy (locked) */
#define KER_USER_CANCELLED       -52     /* user cancelled operations i.e. using ESC key  */

/*
    KIPP Function Prototype Declarations
*/

DWORD    FAR PASCAL CallStorageProc(HIMDESC hID, WORD wOp, WORD wParam,
                                    DWORD dwParam, LPINT lpStat);

int      FAR PASCAL CloseImage(HIMDESC hID);

int      FAR PASCAL CopyImage(HIMDESC hSrcID, HIMDESC hDstID, DWORD dwOpt);

HIMDESC  FAR PASCAL CreateCompatibleImageDesc(HIMDESC hSrcID, LPINT lpStat);

HPRTSPEC FAR PASCAL CreateCompatiblePrintSpec(HPRTSPEC hPrintSpec,
                                              LPINT lpStat);

HSCNSPEC FAR PASCAL CreateCompatibleScanSpec(HSCNSPEC hScanSpec,
                                             LPINT lpStat);

int      FAR PASCAL CreateImage(HIMDESC hID);

HIMDESC  FAR PASCAL CreateImageDesc(FLTPROC lpfnStgProc, LPINT lpStat);

HPRTSPEC FAR PASCAL CreatePrintSpec(LPINT lpStat);

HSCNSPEC FAR PASCAL CreateScanSpec(LPINT lpStat);

DWORD    FAR PASCAL DefStorageProc(void far *lpExt, WORD wOp, WORD wParam,
                                   DWORD dwParam, LPINT lpStat);

int      FAR PASCAL DeleteImageDesc(HIMDESC hID);

int      FAR PASCAL DeletePrintSpec(HPRTSPEC hPrintSpec);

int      FAR PASCAL DeleteScanSpec(HSCNSPEC hScanSpec);

#if defined(OS2)
int      FAR PASCAL DisplayImage(HIMDESC hID, LPRECT lpSrcRect, 
                                 HAB hab, HPS hpsDest,HWND hDestWnd,
                                 PRECTL lpDestRect, PRECTL lpClipRect,
                                 COORD xOffset, COORD yOffset,
                                 LPXYSCALE lpScale, DWORD dwOptions);
#else
int      FAR PASCAL DisplayImage(HIMDESC hID, LPRECT lpSrcRect,
                                #if defined(WIN)
                                 HDC hDestDC, HWND hDestWnd,
                                #elif defined(DOS)
                                 HDC hDestDC,
                                #elif defined(UNIX)
                                 HWINDOW hDestWindow,
                                #endif
                                 LPRECT lpDestRect, LPRECT lpClipRect,
                                 COORD xOffset, COORD yOffset,
                                 LPXYSCALE lpScale, DWORD dwOptions);
#endif

DWORD    FAR PASCAL GetImageAttr(HIMDESC hID, WORD wAttrId, LPINT lpStat);

int      FAR PASCAL GetImageAttrString(HIMDESC hID, WORD wAttrId,
                                       LPSTR lpString, int nMaxCount,
                                       LPINT lpStat);

WORD     FAR PASCAL GetKIPPVersion(void);

WORD     FAR PASCAL GetKIPPRevision(void);

int      FAR PASCAL GetKIPPStatusText(int nStat, LPSTR lpString,
                                      int nMaxCount, LPINT lpStat);

WORD     FAR PASCAL GetResourceCount(void);

DWORD    FAR PASCAL GetResourceInfo(int nResource, WORD wIndex, LPINT lpStat);

int      FAR PASCAL GetResourceInfoString(int nResource, WORD wIndex,
                                          LPSTR lpString, int nMaxCount,
                                          LPINT lpStat);

DWORD    FAR PASCAL GetPrintParam(HPRTSPEC hPrintSpec, WORD wParamId,
                                  LPINT lpStat);

int      FAR PASCAL GetPrintParamIndirect(HPRTSPEC hPrintSpec, WORD wParamId,
                                          LPSTR lpBuf, int nBufSize,
                                          LPINT lpStat);

DWORD    FAR PASCAL GetScanParam(HSCNSPEC hScanSpec, WORD wParamId,
                                 LPINT lpStat);

int      FAR PASCAL GetScanParamIndirect(HSCNSPEC hScanSpec, WORD wParamId,
                                         LPSTR lpBuf, int nBufSize,
                                         LPINT lpStat);

int      FAR PASCAL OpenImage(HIMDESC hID);

KERR_ERR_ENUM FAR PASCAL PrintImage(HIMDESC hID, HPRTSPEC hPrtSpec, DWORD dwOpt);

KERR_ERR_ENUM FAR PASCAL PrinterCommunication(HPRTSPEC hPrtSpec,
                              LPBYTE lpSend, WORD wSendCount,
                              LPBYTE lpReceive, WORD wReceiveCount,
                              WORD wOperation, WORD wTimeout);

LONG     FAR PASCAL ReadImage(HIMDESC hID,
                              char huge *hpBuffer, LONG lSize, WORD wPitch,
                              COORD XSrc, COORD YSrc,
                              WORD wSrcWidth, WORD wSrcLength,
                              LPXYSCALE lpScale, DWORD dwOpt, LPINT lpStat);

int      FAR PASCAL ReadPrinterSettings(HPRTSPEC hPrintSpec);

int      FAR PASCAL ReadScannerSettings(HSCNSPEC hScanSpec);

int      FAR PASCAL RemoveImage(HIMDESC hID);

int      FAR PASCAL ReservePrintEngine(HPRTSPEC hPrintSpec);

#if defined(OS2)
KERR_ERR_ENUM FAR PASCAL ScanDisplayImage(HIMDESC hID, HSCNSPEC hScanSpec,
                                          LPRECT lpSrcRect, 
                                     HAB hab, HPS hpsDest, HWND hDestWnd,
                                     PRECTL lpDestRect, PRECTL lpClipRect,
                                     COORD xOffset, COORD yOffset,
                                     LPXYSCALE lpScale, DWORD dwOpt);
#else
KERR_ERR_ENUM FAR PASCAL ScanDisplayImage(HIMDESC hID, HSCNSPEC hScanSpec,
                                          LPRECT lpSrcRect,
                                    #if defined(WIN)
                                     HDC hDestDC, HWND hDestWnd,
                                    #elif defined(DOS)
                                     HDC hDestDC,
                                    #elif defined(UNIX)
                                     HWINDOW hDestWindow,
                                    #endif
                                     LPRECT lpDestRect, LPRECT lpClipRect,
                                     COORD xOffset, COORD yOffset,
                                     LPXYSCALE lpScale, DWORD dwOpt);
#endif

KERR_ERR_ENUM FAR PASCAL ScanImage(HIMDESC hID, HSCNSPEC hScanSpec, DWORD dwOpt);

KERR_ERR_ENUM FAR PASCAL ScannerCommunication(HSCNSPEC hScanSpec,
                                LPBYTE lpSend, WORD wSendCount,
                                LPBYTE lpReceive, WORD wReceiveCount,
                                WORD wOperation, WORD wTimeout);

DWORD    FAR PASCAL SetImageAttr(HIMDESC hID, WORD wAttrId, DWORD dwValue,
                                LPINT lpStat);
          
DWORD    FAR PASCAL SetPrintParam(HPRTSPEC hPrintSpec, WORD wParamId,
                                DWORD dwValue, LPINT lpStat);

DWORD    FAR PASCAL SetScanParam(HSCNSPEC hScanSpec, WORD wParamId,
                                 DWORD dwValue, LPINT lpStat);

int      FAR PASCAL UnreservePrintEngine(HPRTSPEC hPrintSpec);

LONG     FAR PASCAL WriteImage(HIMDESC hID,
                               char huge *hpBuffer, LONG lSize, WORD wPitch,
                               COORD XDst, COORD YDst,
                               WORD wDstWidth, WORD wDstLength,
                               LPXYSCALE lpScale, DWORD dwOpt, LPINT lpStat);

int      FAR PASCAL WritePrinterSettings(HPRTSPEC hPrintSpec);

int      FAR PASCAL WriteScannerSettings(HSCNSPEC hScanSpec);

typedef KERR_ERR_ENUM   (FAR PASCAL *LPFN_IDLE)
    (
    WORD            wParam,         /* caller's WORD parameter */
    DWORD           dwParam         /* caller's DWORD parameter */
    );

typedef KERR_ERR_ENUM   (FAR PASCAL *LPFN_CLAIM_SCAN_PARAMS)
    (
    HIMDESC FAR     *lphImageDesc,  /* address of returned image descriptor handle */
    LPRECT FAR      *lplpSrcRect,   /* address of the address of the returned source
                                       rectangle */
    #if defined(OS2)
     HAB FAR         *lphab,       
     HPS FAR         *lphpsDest, 
    #endif
    #if defined(WIN) || defined(DOS)
     HDC FAR         *lphDestDC,     /* address of returned device context handle */
    #endif
    #if defined(WIN) || defined(OS2)
     HWND FAR        *lphDestWnd,    /* address of returned window handle */
    #elif defined(UNIX)
     HWINDOW FAR     *lphDestWindow, /* address of returned window handle */
    #endif
    #if defined(WIN) || defined(DOS) || defined(UNIX)
     LPRECT FAR      *lplpDestRect,  /* address of the address of the returned
                                        destination rectangle */    
     LPRECT FAR      *lplpClipRect,  /* address of the address of the returned
                                        clipping rectangle */   
    #else
     PRECTL FAR      *lppDestRect,
     PRECTL FAR      *lppClipRect,
    #endif
    COORD FAR       *lpxOffset,     /* address of the returned x - offset */
    COORD FAR       *lpyOffset,     /* address of the returned y - offset */
    LPXYSCALE FAR   *lplpScale,     /* address of the address of the returned
                                       scale factors */ 
    LPDWORD         lpdwOptions,    /* address of returned scan options */
    WORD            wParam,         /* caller's WORD parameter */
    DWORD           dwParam         /* caller's DWORD parameter */
    );

typedef KERR_ERR_ENUM   (FAR PASCAL *LPFN_RELEASE_SCAN_PARAMS)
    (
    HIMDESC FAR     *lphImageDesc,  /* address of returned image descriptor handle */
    LPRECT FAR      *lplpSrcRect,   /* address of the address of the returned source
                                       rectangle */
    #if defined(OS2)
     HAB FAR         *lphab,       
     HPS FAR         *lphpsDest, 
    #endif
    #if defined(WIN) || defined(DOS)
     HDC FAR         *lphDestDC,     /* address of returned device context handle */
    #endif
    #if defined(WIN) || defined(OS2)
     HWND FAR        *lphDestWnd,    /* address of returned window handle */
    #elif defined(UNIX)
     HWINDOW FAR     *lphDestWindow, /* address of returned window handle */
    #endif
    #if defined(WIN) || defined(DOS) || defined(UNIX)
     LPRECT FAR      *lplpDestRect,  /* address of the address of the returned
                                        destination rectangle */    
     LPRECT FAR      *lplpClipRect,  /* address of the address of the returned
                                        clipping rectangle */   
    #else
     PRECTL FAR      *lppDestRect,
     PRECTL FAR      *lppClipRect,
    #endif
    COORD FAR       *lpxOffset,     /* address of the returned x - offset */
    COORD FAR       *lpyOffset,     /* address of the returned y - offset */
    LPXYSCALE FAR   *lplpScale,     /* address of the address of the returned
                                       scale factors */ 
    LPDWORD         lpdwOptions,    /* address of returned scan options */
    WORD            wParam,         /* caller's WORD parameter */
    DWORD           dwParam,        /* caller's DWORD parameter */
    KERR_ERR_ENUM   nStat           /* status code for scanning operation */
    );

KERR_ERR_ENUM FAR PASCAL    ScanMultipleImages
    (
    HSCNSPEC                    hScanSpec,              /* Handle of the scan spec */
    LPFN_CLAIM_SCAN_PARAMS      lpfnClaimScanParams,    /* function used to 
                                                            get the parameters
                                                            for the next
                                                            scanning operation */
    LPFN_RELEASE_SCAN_PARAMS    lpfnReleaseScanParams,  /* function used to
                                                            release the parameters
                                                            from the previous
                                                            scanning operation */
    LPFN_IDLE       lpfnScanIdle,                       /* idle function */
    WORD            wParam,                             /* caller's WORD parameter */
    DWORD           dwParam                             /* caller's DWORD parameter */
    );

typedef KERR_ERR_ENUM   (FAR PASCAL *LPFN_CLAIM_PRINT_PARAMS)
    (
    HIMDESC FAR     *lphImageDesc,  /* address of returned image descriptor handle */
    LPDWORD         lpdwOptions,    /* address of returned print options */
    WORD            wParam,         /* caller's WORD parameter  */
    DWORD           dwParam         /* caller's DWORD parameter  */
    );

typedef KERR_ERR_ENUM   (FAR PASCAL *LPFN_RELEASE_PRINT_PARAMS)
    (
    HIMDESC FAR     *lphImageDesc,  /* address of image descriptor handle to be released */
    LPDWORD         lpdwOptions,    /* address of print options to be released */
    WORD            wParam,         /* caller's WORD parameter  */
    DWORD           dwParam,        /* caller's DWORD parameter  */
    KERR_ERR_ENUM   nStat           /* status code for printing operation */
    );

KERR_ERR_ENUM FAR PASCAL   PrintMultipleImages
    (
    HPRTSPEC                  hPrintSpec,             /*  print spec handle */
    LPFN_CLAIM_PRINT_PARAMS   lpfnClaimPrintParams,   /*  function used to get
                                                         the parameters for the
                                                         next printing operation */
    LPFN_RELEASE_PRINT_PARAMS lpfnReleasePrintParams, /* function used to release
                                                         the parameters from previous
                                                         printing operation */
    LPFN_IDLE        lpfnPrintIdle,                   /* user idle function */
    WORD             wParam,                          /* user's WORD parameter*/
    DWORD            dwParam                          /* user's DWORD parameter*/
    );

KERR_ERR_ENUM FAR PASCAL KIPPOpen(LPHANDLE lphKipCxt);

KERR_ERR_ENUM FAR PASCAL KIPPClose(HANDLE hKipCxt);

typedef KERR_ERR_ENUM (FAR PASCAL *LPFN_ADJUST_DETECTED_WIDTH)
    (
    LPWORD      lpwWidth,       /* detected width in pixels */
    WORD        wDPI,           /* scanner DPI (actual dots per inch, NOT the
                                    same values as used for KSP_wDPI, i.e.,
                                    this will be 300 NOT DPI_300 for 300 DPI */
    BYTE        byOrientation   /* orientation: PORTRAIT or LANDSCAPE (These
                                    are the same values as used for
                                    KSP_byORIENTATION */
    );      

#ifdef UNIX
/* UNIX specific routines for dealing with multiple adapters */

KERR_ERR_ENUM SetKIPPAdapterNum(int adapter);

int GetKIPPAdapterNum(void);

BOOL KIPPAppReserveAdapter(void);

BOOL KIPPAppReleaseAdapter(void);

#endif /* UNIX specific routines */

#endif      /* kipp_h */
