/*****************************************************************************
*       (c) Copyright Kofax Image Products, 1990.  All rights reserved.
*               Unauthorized use, duplication, or distribution is strictly prohibited.
******************************************************************************
*
*                File:  kos.h
*         Purpose:      Kofax Operating System-Dependent Module
*
*       $Revision:   1.26  $
*               $Date:   15 Oct 1992 10:35:36  $
*
*   $NoAuthor:$
*
*      $NoLog:$
*
*****************************************************************************/

#if !defined(kos_h)
#define kos_h


#if defined (OS2) || defined(WIN) || defined(DOS) || defined(UNIX) 
        #define KOSStrNCpy                      KUTStrNCpy
        #define KOSStrLen                       KUTStrLen
        #define KOSStrCpy                       KUTStrCpy
#endif

/****************************************************************************
/*      UNIX ONLY
*****************************************************************************/

#if defined (UNIX)

/* This is an unfortunate layering of includes.  kgd.h should be
 * included only in those modules that directly call KGD routines,
 * but the definitions of HDISPLAY and HWND are needed in kipp.h.
 */
#include "kgd.h"

#define FARPROC                         PFNWP
#define DSVAR                                   /* no problem passing the address of a   */
                                                                /* stack-based variable as a far pointer */
#define KSHARED /* nil */               /* no need for dedicated shared data seg */
#define wKOSError               ((WORD) errno)

#endif

/****************************************************************************
/*      OS2 ONLY
*****************************************************************************/

#if defined (OS2)
                                                                /* use this hab temporary until the code to  */
                                                                /* obtain one from the caller is implemented */
#define KIPP_HAB                   (HAB)0
#define FARPROC                         PFNWP

#define DSVAR                                   /* no problem passing the address of a   */
                                                                /* stack-based variable as a far pointer */

#define KSHARED FAR                             /* force data in dedicated shared data seg */

#define KOS_EXLST_ADD           EXLST_ADD
#define KOS_EXLST_REMOVE        EXLST_REMOVE
#define KOS_EXLST_EXIT          EXLST_EXIT
#define KOS_TC_EXIT                     TC_EXIT
#define KOS_TC_HARDERROR        TC_HARDERROR
#define KOS_TC_TRAP                     TC_TRAP
#define KOS_TC_KILLPROCESS      TC_KILLPROCESS


WORD FAR PASCAL KOSConvertToKippRect (
        HWND    hWnd,                           /* window in which source rectangle is located */
        HPS         hPS,                                /* PS used if window is NULL */
        LPRECT  lpDstRect,                      /* destination Kipp-format rectangle */
        PRECTL  lpSrcRect);                     /* source rectangle to be converted */

WORD FAR PASCAL KOSConvertToOsRect (
        HWND    hWnd,                           /* window in which rectangle is located */
        HPS         hPS,                                /* PS used if window is NULL */
        PRECTL  lpDstRect,                      /* destination OS-format rectangle */
        LPRECT  lpSrcRect);                     /* source rectangle to be converted */

WORD FAR PASCAL KOSMapToKippRect (
        HWND    hWnd,                           /* window in which source rectangle is located */
        HPS         hPS,                                /* PS used if window is NULL */
        LPRECT  lpDstRect,                      /* destination Kipp-format rectangle */
        PRECTL  lpSrcRect);                     /* source rectangle to be converted */

WORD FAR PASCAL KOSMapToOsRect (
        HWND    hWnd,                           /* window in which rectangle is located */
        HPS         hPS,                                /* PS used if window is NULL */
        PRECTL  lpDstRect,                      /* destination OS-format rectangle */
        LPRECT  lpSrcRect);                     /* source rectangle to be converted */

extern WORD wKOSError;                  /* most recent OS error */

#define KOS_SEM_IMMEDIATE_RETURN  SEM_IMMEDIATE_RETURN
#define KOS_SEM_INDEFINITE_WAIT   SEM_INDEFINITE_WAIT

#endif


/****************************************************************************
/*      MSDOS ONLY
*****************************************************************************/

#if defined (DOS)
#define dosutil_h

typedef VOID (*PKOSEXITFN)(VOID);
typedef int (FAR PASCAL *FARPROC)();
typedef DWORD (FAR PASCAL *FLTPROC)(
                                VOID FAR *lpCxt,
                                WORD  wOp,
                                WORD  wParam,
                                DWORD dwParam,
                                LPINT lpStat);

#define wKOSError                  _doserrno

#define DSVAR                                   /* no problem passing the address of a   */
                                                                /* stack-based variable as a far pointer */

#define KSHARED                                 /* no need for dedicated shared data seg */

/***** Display Filter Functions...  */

/*
        DC -- DISPLAY CONTEXT STRUCTURE

        NOTES:  A display context structure contains information about
            the display.  It consists of a main body defined by this header file
                and an extension defined and maintained by the display filter function
                assigned when the display context is created.  The extension
            size is specified by the display filter.
*/
typedef struct tagDC FAR *LPDC;

typedef struct tagDC
{
        LONG            lMagic;                 /* identifies this struct as DC */
        int                     nExtSize;               /* DC extension size */
        DWORD           dwStatus;               /* status - not sure what this is going to be used for */
        DWORD           dwFlags;                /* display flags: indicates desired color and bitsex for
                                                                        bitmaps */
        FLTPROC lpfnDFProc;                     /* display process (filter) */
} DC;

/***** Magic -- DC Descriptor Magic Number */

#define KDF_MAGIC               0x0063642EL             /* '.df' */

/***** Common (Kofax-supported) Display Filter Operation Codes */

#define KDF_INIT                        1       /* initialize display context */
#define KDF_DELETE                      2       /* delete display context (free resources) */
#define KDF_lGETATTR            3       /* get display attribute */
#define KDF_lSETATTR            4       /* set display attribute */
#define KDF_lTESTATTRBIT        5       /* test bit of display attribute */
#define KDF_lCLRATTRBIT         6       /* clear bit of display attribute */
#define KDF_lSETATTRBIT         7       /* set bit of display attribute */
#define KDF_nGETEXTSIZE         8       /* get display context extension size */
#define KDF_BLTBITMAP           9       /* blt bitmap to display */
#define KDF_CLEARSCREEN         10      /* clear screen */
#define KDF_CLEARRECT           11      /* clear rectangle */

/*
 *      DISPLAY CONTEXT ATTRIBUTES -- attributes for KDF_lGETATTR and KDF_lSETATTR
 *
 *      NOTES:  * some attributes (bit flags) are part of other attributes.
 */
                                                                /** COMMON DISPLAY ATTRIBUTES **/
#define KDA_hTASK                       1       /* owner of the image desc */
#define KDA_dwFLAGS                     2       /* display mode flags */
#define KDA_wHORZRES            3       /* horizontal resolution in pixels */
#define KDA_wVERTRES            4       /* vertical resolution in pixels */
#define KDA_wHORZDPI            5       /* horizontal dots per inch */
#define KDA_wVERTDPI            6       /* vertical dots per inch */
#define KDA_wWIDTHININCHES      7       /* screen width in inches */
#define KDA_wHEIGHTININCHES     8       /* screen height in inches */
#define KDA_wWIDTHINMM          9       /* screen width in millimeters */
#define KDA_wHEIGHTINMM         10      /* screen height in millimeters */
#define KDA_nPLANES                     11      /* number of color planes in frame-buffer memory */
#define KDA_nBITSPERPLANE       12      /* number of bits per pixel in each color plane */

                                                                                /* users may request that defines
                                                                                   be added to the common attributes
                                                                                   section by contacting
                                                                                   Kofax Image Products */

                                                                                /** USER DEFINED ATTRIBUTES **/
#define KDA_tagUSERATTRIBUTES   256             /* start of user defined attributtes */
#define KDA_USER(user_attr)             (KDA_tagUSERATTRIBUTES + (user_attr))

/***** Display Context Pattern definitions - for use with KDF_CLEARSCREEN */
/***** and KDF_CLEARRECT. */

#define KDF_SOLID_WHITE                 0
#define KDF_SOLID_BLACK                 1
#define KDF_CHECKERBOARD                2
#define KDF_VERTICAL_STRIPES    3
#define KDF_HORIZONTAL_STRIPES  4
#define KDF_DIAGONAL_CROSS              5
#define KDF_VERTICAL_CROSS              6

typedef HANDLE HDC;
typedef HANDLE HBITMAP;

typedef struct tagBITMAP
{       short      bmType;
        short      bmWidth;
        short      bmHeight;
        short      bmWidthBytes;
        BYTE       bmPlanes;
        BYTE       bmBitsPixel;
        LPSTR      bmBits;
} BITMAP;
typedef BITMAP *PBITMAP;
typedef BITMAP NEAR *NPBITMAP;
typedef BITMAP FAR *LPBITMAP;

typedef struct tagDFBLT                 /* Display Filter BLT parameter block */
{
        VOID FAR        *lpDC;                  /* ptr to display filter display context (dest) */
        short int       destX;                  /* destination origin */
        short int       destY;                  /* destination origin */
        short int       nWidth;                 /* width of rectangle */
        short int       nHeight;                /* height of rectangle */
        LPBITMAP        lpBitmap;               /* long pointer to BITMAP structure */
        short int       srcX;                   /* source origin */
        short int       srcY;                   /* source origin */
        DWORD           dwROP;                  /* raster operation */
} KDFBLT;

typedef KDFBLT FAR *LPKDFBLT;

/*  Ternary raster operations */
#define SRCCOPY     (DWORD)0x00CC0020   /* dest=source */
#define NOTSRCCOPY  (DWORD)0x00330008   /* dest = (not source) */

HDC FAR PASCAL CreateDC(
  FLTPROC lpfnDProc,                    /* display proc for display context */
  LPINT lpStat);

int FAR PASCAL DeleteDC(
  HDC hDC);

int FAR PASCAL BltBitmapToDC(
  HDC hDC,                                              /* handle to display context (destination) */
  short int     destX,                          /* destination origin */
  short int     destY,                          /* destination origin */
  short int     nWidth,                         /* width of rectangle */
  short int     nHeight,                        /* height of rectangle */
  LPBITMAP lpBitmap,                    /* long pointer to BITMAP structure */
  short int     srcX,                           /* source origin */
  short int     srcY,                           /* source origin */
  DWORD dwROP);                                 /* raster operation */

DWORD FAR PASCAL GetDCAttr(
  HDC hDC,                                              /* handle to display context */
  WORD wAttrId,                                 /* specifies attribute to be returned */
  LPINT lpStat);                                /* far pointer to status number */

DWORD FAR PASCAL SetDCAttr(
  HDC hDC,                                              /* handle to display context */
  WORD wAttrId,                                 /* specifies attribute to be set */
  DWORD dwValue,                                /* attribute value */
  LPINT lpStat);                                /* far pointer to status code */

int FAR PASCAL GetDCAttrString(
  HDC hDC,                                              /* handle to display context */
  WORD wAttrId,                                 /* specifies attribute to be retrieved */
  LPSTR lpString,                               /* buffer to put the string in */
  int nMaxCount,                                /* max chars to be copied to buffer */
  LPINT lpStat);                                /* far pointer to status code */

DWORD FAR PASCAL DefDcProc(
  VOID FAR *lpExt,                              /* far pointer to display filter's context */
  WORD wOp,                                             /* display filter operation code */
  WORD wParam,                                  /* word parameter */
  DWORD dwParam,                                /* dword word parameter */
  LPINT lpStat);                                /* long pointer to status code */

DWORD FAR PASCAL CallDcProc(
  HDC hDC,                                              /* DC Descriptor extension */
  WORD wOp,                                             /* operation code */
  WORD wParam,                                  /* two byte parameter */
  DWORD dwParam,                                /* four byte parameter */
  LPINT lpStat);                                /* pointer to operation status */

#endif



/****************************************************************************
/*      MS WINDOWS ONLY
*****************************************************************************/

#if defined(WIN)

#define DSVAR   static                  /* cannot safely pass the address of a stack */
                                                                /* based variable as a far pointer, force in */
                                                                /* DLL's default data segment */

#define KSHARED                                 /* no need for dedicated shared data seg */

#define KOSLocalAlloc                   LocalAlloc
#define KOSLocalFree                    LocalFree
#define KOSLocalLock                    LocalLock
#define KOSLocalUnlock                  LocalUnlock
#define KOSLocalFlags                   LocalFlags
#define KOSLocalSize                    LocalSize
#define KOSLocalHandle                  LocalHandle
#define KOSGlobalAlloc                  GlobalAlloc
#define KOSGlobalReAlloc                GlobalReAlloc
#define KOSGlobalFree                   GlobalFree
#define KOSGlobalLock                   GlobalLock
#define KOSGlobalUnlock                 GlobalUnlock
#define KOSGlobalFlags                  GlobalFlags
#define KOSGlobalSize                   GlobalSize
#define KOSSetRect                              SetRect
#define KOSCopyRect(r1,r2)              (CopyRect(r1,r2), 0)
#define KOSIntersectRect                IntersectRect
#define KOSOffsetRect                   OffsetRect
#define KOSIsRectEmpty                  IsRectEmpty
#define KOSGetTickCount                 GetTickCount
#define KOSOpenFile                             OpenFile
#define KOSOpen                                 _lopen
#define KOSClose                                _lclose
#define KOSCreate                               _lcreat
#define KOSSeek                                 _llseek
#define KOSDelete                               _ldelete
#define KOSDup                                  _ldup
#define KOSRead                                 _lread
#define KOSWrite                                _lwrite
#define KOSOpenPathName                 OpenPathname
#define KOSDeletePathName               DeletePathname

typedef struct tagOFSTRUCT              KOS_OFSTRUCT;
typedef KOS_OFSTRUCT               *PKOS_OFSTRUCT;
typedef KOS_OFSTRUCT NEAR          *NPKOS_OFSTRUCT;
typedef KOS_OFSTRUCT FAR           *LPKOS_OFSTRUCT;

#define KOS_LMEM_FIXED                  LMEM_FIXED
#define KOS_LMEM_MOVEABLE               LMEM_MOVEABLE
#define KOS_LMEM_NOCOMPACT              LMEM_NOCOMPACT
#define KOS_LMEM_NODISCARD              LMEM_NODISCARD
#define KOS_LMEM_ZEROINIT               LMEM_ZEROINIT
#define KOS_LMEM_MODIFY                 LMEM_MODIFY
#define KOS_LMEM_DISCARDABLE    LMEM_DISCARDABLE
#define KOS_LHND                                LHND
#define KOS_LPTR                                LPTR
#define KOS_NONZEROLHND                 NONZEROLHND
#define KOS_NONZEROLPTR                 NONZEROLPTR
#define KOS_LNOTIFY_OUTOFMEM    LNOTIFY_OUTOFMEM
#define KOS_LNOTIFY_MOVE                LNOTIFY_MOVE
#define KOS_LNOTIFY_DISCARD             LNOTIFY_DISCARD
#define KOS_LMEM_DISCARDED              LMEM_DISCARDED
#define KOS_LMEM_LOCKCOUNT              LMEM_LOCKCOUNT

#define KOS_GMEM_FIXED          GMEM_FIXED
#define KOS_GMEM_MOVEABLE               GMEM_MOVEABLE
#define KOS_GMEM_NOCOMPACT              GMEM_NOCOMPACT
#define KOS_GMEM_NODISCARD              GMEM_NODISCARD
#define KOS_GMEM_ZEROINIT               GMEM_ZEROINIT
#define KOS_GMEM_MODIFY         GMEM_MODIFY
#define KOS_GMEM_DISCARDABLE    GMEM_DISCARDABLE
#define KOS_GHND                                GHND
#define KOS_GPTR                                GPTR
#define KOS_GMEM_SHARE                  GMEM_SHARE
#define KOS_GMEM_DDESHARE               GMEM_DDESHARE
#define KOS_GMEM_NOT_BANKED     GMEM_NOT_BANKED
#define KOS_GMEM_NOTIFY                 GMEM_NOTIFY
#define KOS_GMEM_LOWER                  GMEM_LOWER
#define KOS_GMEM_DISCARDED              GMEM_DISCARDED
#define KOS_GMEM_LOCKCOUNT              GMEM_LOCKCOUNT

#define KOS_OF_READ                     OF_READ
#define KOS_OF_WRITE                    OF_WRITE
#define KOS_OF_READWRITE                OF_READWRITE
#define KOS_OF_PARSE                    OF_PARSE
#define KOS_OF_DELETE                   OF_DELETE
#define KOS_OF_VERIFY                   OF_VERIFY
#define KOS_OF_CANCEL                   OF_CANCEL
#define KOS_OF_CREATE                   OF_CREATE
#define KOS_OF_PROMPT                   OF_PROMPT
#define KOS_OF_EXIST                    OF_EXIST
#define KOS_OF_REOPEN                   OF_REOPEN

/*
        WINDOWS undocumented KERNEL functions
*/
int   FAR PASCAL DeletePathname(LPSTR);
#ifndef WIN31
/*
        These functions are now documented in windows and no longer need
        be defined here.  JDH
*/
//int   FAR PASCAL _lopen(LPSTR, int);
int   FAR PASCAL _lclose(int);
//int   FAR PASCAL _lcreat(LPSTR, int);
LONG  FAR PASCAL _llseek(int, long, int);
//WORD  FAR PASCAL _lread(int, LPSTR, int);
//WORD  FAR PASCAL _lwrite(int, LPSTR, int);
BOOL  FAR PASCAL _ldelete(LPSTR);
int   FAR PASCAL OpenPathname(LPSTR, int);
#endif
#endif


/****************************************************************************
/*      WINDOWS, OS2, UNIX
*****************************************************************************/

#if defined (WIN) || defined (OS2) || defined (UNIX)

typedef VOID (PASCAL FAR *PKOSEXITFN)(WORD);

#endif

/****************************************************************************
/*      MSDOS, WINDOWS, UNIX
*****************************************************************************/

#if defined (DOS) || defined (WIN) || defined (UNIX)

typedef WORD                                      PID;
typedef WORD                                      TID;

#define KOS_SEM_IMMEDIATE_RETURN   0L
#define KOS_SEM_INDEFINITE_WAIT   -1L

#define KOS_EXLST_ADD                   1
#define KOS_EXLST_REMOVE                2
#define KOS_EXLST_EXIT                  3
#define KOS_TC_EXIT                     0
#define KOS_TC_HARDERROR                1
#define KOS_TC_TRAP                     2
#define KOS_TC_KILLPROCESS              3

#endif


/****************************************************************************
/*      MSDOS, OS2, UNIX
*****************************************************************************/

#if defined (DOS) || defined (OS2) || defined (UNIX)

typedef HANDLE             *PHANDLE;
typedef HANDLE  NEAR   *SPHANDLE;
typedef HANDLE  FAR        *LPHANDLE;

typedef HANDLE          GLOBALHANDLE;
typedef HANDLE          LOCALHANDLE;

#define MAKE_FAR_PTR(segment, offset)   (char FAR *) MAKELONG(offset, segment)
#if !defined(LOWORD)
        #define LOWORD(l)               ((WORD)((DWORD) l))
#endif
#if !defined(HIWORD)
        #define HIWORD(l)               ((WORD)(((DWORD)(l) >> 16) & 0xFFFF))
#endif

/***** Points...***** Moved to kcommon.h ***** */

/* typedef struct tagPOINT
/* {   int x;
/*      int y;
/* } POINT;
/* typedef POINT                   *PPOINT;
/* typedef POINT NEAR    *NPPOINT;
/* typedef POINT FAR       *LPPOINT;
/* 
/* #define MAKEPOINT(l) (*((POINT *)&(l))) */

struct tagKOS_OFSTRUCT
{       BYTE cBytes;                            /* length of structure */
    BYTE fFixedDisk;                    /* non-zero if file located on non- */
                                                                /* removeable media */
    WORD nErrCode;                              /* OS error code if OpenFile fails */
    BYTE reserved[4];
    BYTE szPathName[128];
 };
typedef struct tagKOS_OFSTRUCT KOS_OFSTRUCT;
typedef KOS_OFSTRUCT       *PKOS_OFSTRUCT;
typedef KOS_OFSTRUCT NEAR *NPKOS_OFSTRUCT;
typedef KOS_OFSTRUCT FAR  *LPKOS_OFSTRUCT;


#if defined(DOS)
/***** Local memory is supported only for backwards compatibility with */
/***** previous versions of KIPP.  It is recommended that Global memory */
/***** functions be used instead. */

/***** Interface to local memory manager */

#define KOS_LMEM_FIXED              0x0000
#define KOS_LMEM_MOVEABLE           0x0002
#define KOS_LMEM_NOCOMPACT          0x0010
#define KOS_LMEM_NODISCARD          0x0020
#define KOS_LMEM_ZEROINIT           0x0040
#define KOS_LMEM_MODIFY                 0x0080
#define KOS_LMEM_DISCARDABLE    0x0F00
#define KOS_LHND                                (KOS_LMEM_MOVEABLE | KOS_LMEM_ZEROINIT)
#define KOS_LPTR                                (KOS_LMEM_FIXED    | KOS_LMEM_ZEROINIT)
#define KOS_NONZEROLHND                 (KOS_LMEM_MOVEABLE)
#define KOS_NONZEROLPTR                 (KOS_LMEM_FIXED)

#define KOS_LNOTIFY_OUTOFMEM    0
#define KOS_LNOTIFY_MOVE                1
#define KOS_LNOTIFY_DISCARD             2

/***** Flags returned by LocalFlags (in addition to KOS_LMEM_DISCARDABLE) */

#define KOS_LMEM_DISCARDED          0x4000
#define KOS_LMEM_LOCKCOUNT          0x00FF

/***** Local memory functions... */

HANDLE FAR PASCAL KOSLocalAlloc(
  WORD wFlags,                                  /* specifies how to allocate memory */
  WORD wBytes);                                 /* Total number of bytes to be allocated */

WORD FAR PASCAL KOSLocalFree(
  HANDLE hMem);                                 /* handle to local memory block to be freed */

char NEAR * FAR PASCAL KOSLocalLock(
  HANDLE hMem);                                 /* handle of local memory block to be locked */

BOOL FAR PASCAL KOSLocalUnlock(
  HANDLE hMem);                                 /* handle of local memory block to be locked */

WORD FAR PASCAL KOSLocalFlags(
  HANDLE hMem);                                 /* handle to local memory block */

WORD FAR PASCAL KOSLocalSize(
  HANDLE hMem);                                 /* handle to local memory block */

HANDLE FAR PASCAL KOSLocalHandle(
        WORD wMem                                       /* handle or pointer to local memory block */
        );
#endif

/***** Interface to global memory manager */

#define KOS_GMEM_FIXED          0x0000
#define KOS_GMEM_MOVEABLE           0x0002
#define KOS_GMEM_NOCOMPACT          0x0010
#define KOS_GMEM_NODISCARD          0x0020
#define KOS_GMEM_ZEROINIT           0x0040
#define KOS_GMEM_MODIFY         0x0080
#define KOS_GMEM_DISCARDABLE    0x0F00
#define KOS_GHND                                (KOS_GMEM_MOVEABLE | KOS_GMEM_ZEROINIT)
#define KOS_GPTR                                (KOS_GMEM_FIXED    | KOS_GMEM_ZEROINIT)
#define KOS_GMEM_SHARE                  0x2000
#define KOS_GMEM_DDESHARE           0x2000
#define KOS_GMEM_NOT_BANKED     0x1000
#define KOS_GMEM_NOTIFY             0x4000
#define KOS_GMEM_LOWER              KOS_GMEM_NOT_BANKED

/***** Flags returned by GlobalFlags (in addition to KOS_GMEM_DISCARDABLE) */

#define KOS_GMEM_DISCARDED          0x4000
#define KOS_GMEM_LOCKCOUNT          0x00FF


/***** Global memory functions... */

HANDLE FAR PASCAL KOSGlobalAlloc(
  WORD wFlags,                                  /* specifies how to allocate memory */
  DWORD dwBytes);                               /* Total number of bytes to be allocated */

HANDLE FAR PASCAL KOSGlobalReAlloc(
  HANDLE hMem,                                  /* handle to memory block */
  DWORD dwBytes,                                /* New size in bytes */
  WORD wFlags);                                 /* how to reallocate the global memory block */

HANDLE FAR PASCAL KOSGlobalFree(
  HANDLE hMem);                                 /* handle to global memory block to be freed */

char FAR * FAR PASCAL KOSGlobalLock(
  HANDLE hMem);                                 /* handle of global memory block to be locked */

BOOL FAR PASCAL KOSGlobalUnlock(
  HANDLE hMem);                                 /* handle of global memory block to be unlocked */

WORD FAR PASCAL KOSGlobalFlags(
  HANDLE hMem);                                 /* handle to global memory block */

DWORD FAR PASCAL KOSGlobalSize(
  HANDLE hMem);                                 /* handle to global memory block */

VOID PASCAL KOSGlobalMemoryCleanup(
        VOID);


/***** Rectangle functions... */

int     FAR PASCAL KOSSetRect(
  LPRECT lpRect,                                /* destination rectangle */
  int x1,                                               /* coordinates of upper-left corner */
  int y1,
  int x2,                                               /* coordinates of lower-right corner */
  int y2);

int FAR PASCAL KOSCopyRect(
  LPRECT lpDestRect,                    /* destination rectangle */
  LPRECT lpSrcRect);                    /* source rectangle */

int     FAR PASCAL KOSIntersectRect(
  LPRECT lpDestRect,                    /* destination rectangle */
  LPRECT lpSrc1Rect,                    /* source rectangle */
  LPRECT lpSrc2Rect);                   /* source rectangle */

int     FAR PASCAL KOSOffsetRect(
  LPRECT lpRect,                                /* destination rectangle */
  int xOffset,                                  /* x offset */
  int yOffset);                                 /* y offset */

BOOL FAR PASCAL KOSIsRectEmpty(
  LPRECT lpRect);                               /* rectangle */




WORD FAR PASCAL KOSCommOut (
 LPSTR lpBuf);                                  /* pointer to ASCIIZ string */

WORD FAR PASCAL KOSMonoOut (
 LPSTR lpBuf);                                  /* pointer to ASCIIZ string */

WORD FAR PASCAL KOSPrnOut  (
 LPSTR lpBuf);                                  /* pointer to ASCIIZ string */


/***** Time function... */

DWORD FAR PASCAL KOSGetTickCount(VOID);


/***** File Access Functions... */

int     FAR PASCAL KOSOpenFile(
  LPSTR lpszPathName,                   /* long pointer to path name */
  LPKOS_OFSTRUCT lpReOpenOFS,   /* long pointer to caller's ReOpen OFSTRUCT */
  WORD wAction);                                /* the action to take */


#define KOS_OF_READ          0x0000     /* open for read only */
#define KOS_OF_WRITE     0x0001 /* open for write only */
#define KOS_OF_READWRITE 0x0002 /* open for reading and writing */
#define KOS_OF_PARSE     0x0100 /* parse path name and fill in *lpReOpenOFS */
#define KOS_OF_DELETE    0x0200 /* delete the file */
#define KOS_OF_VERIFY    0x0400 /* verify data/time of file on re-open */
#define KOS_OF_CANCEL    0x0800 /* add a Cancel button to the dialog box */
#define KOS_OF_CREATE    0x1000 /* create new file or truncate existing file */
#define KOS_OF_PROMPT    0x2000 /* display dialog box to insert the diskette */
#define KOS_OF_EXIST     0x4000 /* open and the close the file */
#define KOS_OF_REOPEN    0x8000 /* use file name im *lpReOpenOFS */

int FAR PASCAL KOSOpen(
  LPSTR lpPathName,                             /* far pointer to file's path name. */
  int   mode);                                  /* mode for opening the file */

int FAR PASCAL KOSClose(
  int hFile);                                   /* OS file handle */

int FAR PASCAL KOSCreate(
  LPSTR lpPathName,                             /* far pointer to file's path name */
  int attributes);                              /* attributes for creating the file:
                                                                                0       - normal
                                                                                1       - read-only
                                                                                2       - hidden
                                                                                4       - system */
int FAR PASCAL KOSRead(
  int hFile,                                    /* OS file handle */
  LPSTR lpBuffer,                               /* far pointer to buffer */
  int nBytes);                                  /* number of bytes to be read */

int FAR PASCAL KOSWrite(
  int hFile,                                    /* OS file handle */
  LPSTR lpBuffer,                               /* far pointer to buffer */
  int nBytes);                                  /* number of bytes to be written */

LONG FAR PASCAL KOSSeek(
  int hFile,                                    /* OS file handle */
  long dwOffset,                                /* long offset from origin */
  int wOrigin);                                 /* origin of seek:
                                                                        0 - seek from beginning of file
                                                                        1 - seek from current location of file ptr
                                                                        2 - seek from end of file */
BOOL FAR PASCAL KOSDelete(
  LPSTR lpPathName);                    /* far pointer to file's path name. */

int FAR PASCAL KOSDup(
  int hFile);                                   /* OS file handle */

int FAR PASCAL KOSOpenPathName(
  LPSTR lpszPathname,                   /* pathname of file to be opened */
  int wAccessCode);                             /* desired access/action */

int FAR PASCAL KOSDeletePathName(
  LPSTR lpszPathname);                  /* pathname of file to be deleted */

#endif



/****************************************************************************
/*      WINDOWS, OS2, MSDOS, UNIX
*****************************************************************************/

                                                                /* FLAGS FOR KOSOpen() */
#define KOS_READ           0    /* open for read only */
#define KOS_WRITE          1    /* open for write only */
#define KOS_READWRITE      2    /* open for reading and writing */

                                                                /* FLAGS FOR KOSSeek() */
#define KOS_SEEK_SET       0    /* beginning of file */
#define KOS_SEEK_CUR       1    /* current position of file pointer */
#define KOS_SEEK_END       2    /* end of file */

/***** Semaphores */

typedef struct tagKOSSEM
 {      PID     pid;
        TID     tid;
        HANDLE  hTask;
        DWORD   dwSem;
 }  KOSSEM, FAR *LPKOSSEM;

KERR_ERR_ENUM FAR PASCAL KOSSemRequest(
        LPKOSSEM lpKOSSem,                      /* semaphore to be claimed */
        HANDLE   hTask,                         /* requesting task */
        LONG     lTimeout);                     /* time to wait (ms) for semaphore */

KERR_ERR_ENUM FAR PASCAL KOSSemRelease(
        LPKOSSEM lpKOSSem,                      /* semaphore to be released */
        HANDLE   hTask,                         /* requesting task */
        BOOL     bAppTerminate);        /* flag, set if app is being terminated */

KERR_ERR_ENUM FAR PASCAL KOSEnterCritSec(
        VOID);

KERR_ERR_ENUM FAR PASCAL KOSExitCritSec (
        VOID);


/***** Exit List Processing */

KERR_ERR_ENUM FAR PASCAL KOSExitList(
        WORD             wFunctionCode, /* function code */
        PKOSEXITFN   pfnTermination     /* pointer to termination function */
        );


/***** Processes and Threads */

KERR_ERR_ENUM FAR PASCAL KOSGetProcessInfo (
        PID FAR *lpPID,                         /* points to variable to receive PID */
        TID FAR *lpTID);                        /* points to variable to receive TID */

KERR_ERR_ENUM FAR PASCAL KOSSleep(
        DWORD dwTime);                          /* suspend processing for dwTime millisecs */


/***** Temporary Files */

KERR_ERR_ENUM FAR PASCAL KOSCreateTmpFile(
        LPSTR lpszPath);                        /* buffer for resulting file name */




/****************************************************************************
                        OS2, MSDOS, UNIX - COMPATIBILITY PROTOTYPES AND DEFINITIONS
  (TO COMPILE SOURCE FILES NOT YET MODIFIED FOR KOFAX KOS NAMING CONVENTION)
*****************************************************************************/

#if defined(OS2) || defined(DOS) || defined(UNIX)
#if defined(KOSCOMP)


/***** Compatibility prototypes */

#define LocalAlloc                              KOSLocalAlloc
#define LocalFree                               KOSLocalFree
#define LocalLock                               KOSLocalLock
#define LocalUnlock                             KOSLocalUnlock
#define LocalFlags                              KOSLocalFlags
#define LocalSize                               KOSLocalSize
#define LocalHandle                             KOSLocalHandle
#define GlobalAlloc                             KOSGlobalAlloc
#define GlobalReAlloc                   KOSGlobalReAlloc
#define GlobalFree                              KOSGlobalFree
#define GlobalLock                              KOSGlobalLock
#define GlobalUnlock                    KOSGlobalUnlock
#define GlobalFlags                             KOSGlobalFlags
#define GlobalSize                              KOSGlobalSize
#define SetRect                                 KOSSetRect
#define CopyRect                                KOSCopyRect
#define IntersectRect                   KOSIntersectRect
#define OffsetRect                              KOSOffsetRect
#define IsRectEmpty                             KOSIsRectEmpty
#define GetTickCount                    KOSGetTickCount
#define OpenFile                                KOSOpenFile
#define _lopen                                  KOSOpen
#define _lclose                                 KOSClose
#define _lcreat                                 KOSCreate
#define _llseek                                 KOSSeek
#define _ldelete                                KOSDelete
#define _ldup                                   KOSDup
#define _lread                                  KOSRead
#define _lwrite                                 KOSWrite
#define OpenPathName                    KOSOpenPathName
#define DeletePathname                  KOSDeletePathName


/***** Compatibility definitions */

typedef KOS_OFSTRUCT                    OFSTRUCT;
typedef OFSTRUCT                           *POFSTRUCT;
typedef OFSTRUCT NEAR              *NPOFSTRUCT;
typedef OFSTRUCT FAR               *LPOFSTRUCT;

#define LMEM_FIXED                      KOS_LMEM_FIXED
#define LMEM_MOVEABLE           KOS_LMEM_MOVEABLE
#define LMEM_NOCOMPACT          KOS_LMEM_NOCOMPACT
#define LMEM_NODISCARD          KOS_LMEM_NODISCARD
#define LMEM_ZEROINIT           KOS_LMEM_ZEROINIT
#define LMEM_MODIFY                             KOS_LMEM_MODIFY
#define LMEM_DISCARDABLE        KOS_LMEM_DISCARDABLE
#define LHND                                    KOS_LHND
#define LPTR                                    KOS_LPTR
#define NONZEROLHND                             KOS_NONZEROLHND
#define NONZEROLPTR                             KOS_NONZEROLPTR
#define LNOTIFY_OUTOFMEM                KOS_LNOTIFY_OUTOFMEM
#define LNOTIFY_MOVE                    KOS_LNOTIFY_MOVE
#define LNOTIFY_DISCARD                 KOS_LNOTIFY_DISCARD
#define LMEM_DISCARDED          KOS_LMEM_DISCARDED
#define LMEM_LOCKCOUNT          KOS_LMEM_LOCKCOUNT


#define GMEM_FIXED                      KOS_GMEM_FIXED
#define GMEM_MOVEABLE           KOS_GMEM_MOVEABLE
#define GMEM_NOCOMPACT          KOS_GMEM_NOCOMPACT
#define GMEM_NODISCARD          KOS_GMEM_NODISCARD
#define GMEM_ZEROINIT           KOS_GMEM_ZEROINIT
#define GMEM_MODIFY                     KOS_GMEM_MODIFY
#define GMEM_DISCARDABLE        KOS_GMEM_DISCARDABLE
#define GHND                                    KOS_GHND
#define GPTR                                    KOS_GPTR
#define GMEM_SHARE                              KOS_GMEM_SHARE
#define GMEM_DDESHARE           KOS_GMEM_DDESHARE
#define GMEM_NOT_BANKED         KOS_GMEM_NOT_BANKED
#define GMEM_NOTIFY                     KOS_GMEM_NOTIFY
#define GMEM_LOWER                      KOS_GMEM_LOWER
#define GMEM_DISCARDED          KOS_GMEM_DISCARDED
#define GMEM_LOCKCOUNT          KOS_GMEM_LOCKCOUNT

#define OF_READ                         KOS_OF_READ
#define OF_WRITE                        KOS_OF_WRITE
#define OF_READWRITE                    KOS_OF_READWRITE
#define OF_PARSE                                KOS_OF_PARSE
#define OF_DELETE                               KOS_OF_DELETE
#define OF_VERIFY                               KOS_OF_VERIFY
#define OF_CANCEL                               KOS_OF_CANCEL
#define OF_CREATE                               KOS_OF_CREATE
#define OF_PROMPT                               KOS_OF_PROMPT
#define OF_EXIST                                KOS_OF_EXIST
#define OF_REOPEN                               KOS_OF_REOPEN

/***** End compatibility prototypes and definitions */

#endif
#endif /* NOKOS */



/*****************************************************************************
* These are macros for loading DLL's under Windows and OS/2, and dummying it 
* under DOS.  Use the macros, DO NOT use the functions directly.
*
* KOSLoadLib(hLib, lpszLibName)
*       Returns:        KERR_ERR_ENUM under WIN, OS2, and DOS
*       Note:           Except under DOS, hLib is set (but hLib not &hLib is provided)
* KOSGetProcAddress(hLib, fnFuncName)
*       Returns:        FARPROC under WIN, PFN under OS2, function address under DOS
*       Note:           Do NOT quote the function name, e.g.
*                                       Good:           lpF = KOSGetProcAddress(hLib,MyFunc);
*                                       Bad:            lpF = KOSGetProcAddress(hLib,"MyFunc");
* KOSFreeLib(hLib)
*       Returns:        None
*****************************************************************************/

#if defined(WIN)
        #define KOSGetProcAddress(hLib, fnFuncName)     \
                                GetProcAddress(hLib, (LPSTR) #fnFuncName)
        #define KOSLoadLib(hLib, lpszLibName)   \
                                KOSCvtLibRtn((hLib = LoadLibrary( lpszLibName)))
        #define KOSFreeLib(hLib)                                \
                                (FreeLibrary(hLib), hLib = 0)
#elif defined(OS2)
        #define KOSGetProcAddress(hLib, fnFuncName)     \
                                KOSGetOs2ProcAddr( hLib, (LPSTR) #fnFuncName)
        #define KOSLoadLib(hLib, lpszLibName)   \
                                KOSLoadOs2Dll( lpszLibName, (LPHANDLE) &(hLib))
        #define KOSFreeLib(hLib)                                \
                                (DosFreeModule(hLib), hLib = 0)
#else /* DOS or UNIX */
        #define KOSGetProcAddress(hLib, fnFuncName)             fnFuncName
        #define KOSLoadLib(hLib, lpszLibName)                   SUCCESS
        #define KOSFreeLib(hLib)
#endif

#if defined(WIN)

/*****************************************************************************
* Function:     KOSCvtLibRtn
* Purpose:      Takes the returned handle from a Windows LoadLibrary() call and
*                       determines the return status appropriate for the handle value.
* Returns:
*       SUCCESS,
*       KVEERR_DLL_FILE_NOT_FOUND,              ==> The DLL file was not found
*       KVEERR_DLL_INSUFFICIENT_MEM,    ==> Insufficient memory to load DLL
*       KVEERR_DLL_INVALID_FMT,                 ==> The DLL file's format was bad
*       KVEERR_DLL_UNKNOWN_ERR,                 ==> Unable to load DLL for unknown reason
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL KOSCvtLibRtn
(
        HANDLE  hLib            /* (I) handle to loaded library */
);

#elif defined(OS2)

/*****************************************************************************
* Function:     KOSLoadOs2Dll
* Purpose:      Provides a wrapper around the OS/2 DosLoadModule() function to
*                       make it work like the LoadLibrary()/KOSCvtLibRtn() function pair
*                       used in Windows.
* Returns:
*       SUCCESS,
*       KERR_DLL_FILE_NOT_FOUND,                ==> The DLL file was not found
*       KERR_DLL_INSUFFICIENT_MEM,              ==> Insufficient memory to load DLL
*       KERR_DLL_INVALID_FMT,                   ==> The DLL file's format was bad
*       KERR_DLL_UNKNOWN_ERR,                   ==> Unable to load DLL for unknown reason
*****************************************************************************/
KERR_ERR_ENUM FAR PASCAL KOSLoadOs2Dll
        (
        LPSTR           lpszDllName,    /* (I) The name of the DLL to load */
        LPHANDLE        lphLib                  /* (O) Handle to the loaded DLL */
        );

/*****************************************************************************
* Function:     KOSGetOs2ProcAddr
* Purpose:      Provides a wrapper around the OS/2 DosGetProcAddr() function to
*                       make it work like the GetProcAddress() function used in Windows.
* Returns:
*       SUCCESS,
*       KERR_DLL_CANT_GET_PROC,                 ==> Unable to get procedure address
*       KERR_DLL_NOT_PREV_LOADED,               ==> DLL handle invalid, load probably failed
*       KERR_DLL_UNKNOWN_ERR,                   ==> Unable to get addr for unknown reason
*****************************************************************************/
PFN FAR PASCAL KOSGetOs2ProcAddr
        (
        HANDLE          hLib,                   /* (I) handle to loaded library */
        LPSTR           lpszProcName    /* (I) The name of the DLL to load */
        );

#endif  /* OS */

/* SAFE_SEX platforms have been modified to eliminate the direct copy back
 * of completed DPCBs into user space at interrupt time.  Now implemented
 * on all platforms ... */

# define SAFE_SEX

#endif /* kos_h */
