#ifndef __TKOFAXW_H__
#define __TKOFAXW_H__

#include "dir.h"
#include "owl.h"
#include "control.h"
extern "C" {
#include "kippw.h"
}

#define TKOFAXFILTER_MIN	1
#define TKOFAXFILTER_PDA	1
#define TKOFAXFILTER_TIFF	2
#define TKOFAXFILTER_EMS	3
#define TKOFAXFILTER_PCX	4
#define TKOFAXFILTER_IMG	5
#define TKOFAXFILTER_MAX	5


extern "C" {
WORD	GetFilterType(LPSTR AFileName);
BOOL	IsValidFilterType(WORD AFilterType);

	// Instant Kofax functions

int FAR PASCAL _export	InstantDisplayImage(LPSTR AFileName, LPRECT lpSrcRect, HDC hDestDC, HWND hWnd, LPRECT lpDestRect, LPRECT lpClipRect, int xOffset, int yOffset, LPXYSCALE lpScale, DWORD dwOpt);
DWORD FAR PASCAL _export InstantGetImageAttr(LPSTR AFileName, WORD wAttrId);
int FAR PASCAL _export InstantPrintImage(LPSTR AFileName, DWORD dwOpt);
int FAR PASCAL _export InstantScanImage(LPSTR AFileName, DWORD dwOpt);
}

_CLASSDEF(TKofax)
class _EXPORT TKofax
{
    protected:
	HANDLE		hLib;
	FARPROC		lpfnStgProc;
	HIMDESC		hID;
	int		ErrorCode;
	WORD		wFilterType;

	HPRTSPEC	hPrintSpec;

	HSCNSPEC	hScanSpec;

	int 	 	ErrorMessage(WORD wType = MB_OK);
	int 	 	ErrorMessage(LPSTR lpszMessage, WORD wType = MB_OK);

    public:
	char		FileName[MAXPATH];
	DWORD		dwDisplayOptions;

	TKofax(WORD AFilterType);
	TKofax(LPSTR AImageFile);

		// Library management functions

	void		FreeFilter();
	void 		FreeImageLibrary();
	BOOL		LoadFilter();
	BOOL 		LoadImageLibrary();

        	// Objectized Kofax functions 

	int		KofaxCloseImage();
	int		KofaxCreateImage();
	int		KofaxCreateImageDesc();
	HPRTSPEC 	KofaxCreatePrintSpec();
	HSCNSPEC	KofaxCreateScanSpec();
	int		KofaxDeleteImageDesc();
	int		KofaxDeletePrintSpec();
	int		KofaxDeleteScanSpec();
	int 		KofaxDisplayImage(LPRECT lpSrcRect, HDC hDestDC, HWND hWnd, LPRECT lpDestRect, LPRECT lpClipRect, int xOffset, int yOffset, LPXYSCALE lpScale, DWORD dwOpt);
	DWORD		KofaxGetImageAttr(WORD Attribute);
        DWORD		KofaxGetScanParam(WORD wParamId);
	int		KofaxOpenImage();
	int		KofaxPrintImage(DWORD dwOpt);
	int		KofaxReadPrinterSettings();
	int		KofaxReadScannerSettings();
	int		KofaxReservePrintEngine();
	int		KofaxScanImage(DWORD dwOpt = KSC_NONE);
	DWORD		KofaxSetImageAttr(WORD wAttrId, DWORD dwValue);
	DWORD		KofaxSetPrintParam(WORD wParamId, DWORD dwValue);
	DWORD		KofaxSetScanParam(WORD wParamId, DWORD dwValue);
	int		KofaxUnreservePrintEngine();
	int		KofaxWritePrinterSettings();
	int		KofaxWriteScannerSettings();

		// Simplified Kofax functions

	int 		TKofaxDisplayImage(LPRECT lpSrcRect, HDC hDestDC, HWND hWnd, LPRECT lpDestRect, LPRECT lpClipRect, int xOffset, int yOffset, LPXYSCALE lpScale, DWORD dwOpt);
	DWORD		TKofaxGetImageAttr(WORD wAttrId);
	int		TKofaxPrintImage(DWORD dwOpt);
	int		TKofaxReleaseScanner();
	int		TKofaxReservePrintEngine();
	int		TKofaxScanImage(DWORD dwOpt = KSC_NONE);
        int		TKofaxSetupScanner();
	int		TKofaxUnreservePrintEngine();
};

extern "C" {
BOOL 	RegisterDisplayClass(HINSTANCE hInstance);
int FAR PASCAL _export GetVersionKofaxDisplay();
}

#define	WM_TKOFAX	WM_USER + 0x1000

#define TKD_SETIMAGEFILE	WM_TKOFAX + 1
#define TKD_SETDISPLAYOPTIONS	WM_TKOFAX + 2
#define TKD_SETIMAGEXYPOS	WM_TKOFAX + 3
#define TKD_SETXYSCALE		WM_TKOFAX + 4
#define TKD_GETIMAGEXYPOS	WM_TKOFAX + 5
#define TKD_GETXYSCALE		WM_TKOFAX + 6
#define TKD_GETDISPLAYOPTIONS	WM_TKOFAX + 7
#define TKD_PROMPTXYSCALE	WM_TKOFAX + 8
#define TKD_PROMPTDISPLAYOPTIONS	WM_TKOFAX + 9

_CLASSDEF(TKofaxDisplay)
class _EXPORT TKofaxDisplay : public TControl
{
    protected:
	BOOL		bInDisplay, bValidImage, bButtonDown;
	char		ImageFile[MAXPATH];
	DWORD		dwDisplayOptions;
	XYSCALE		Scale, ScaleHold;
	int		nXPosScrollTo, nYPosScrollTo;
	POINT		ptBeg, ptEnd;
	int		nImageWidth, nImageLength;
	HDC 		HoldDC;

    public:
	TKofaxDisplay(PTWindowsObject AParent, int AnId,  LPSTR ATitle,
		      int X, int Y, int W, int H, PTModule AModule = NULL);
	TKofaxDisplay(PTWindowsObject AParent, int ResourceId, PTModule AModule = NULL);
	LPSTR 		GetClassName(void);
	DWORD		GetDisplayOptions();
	DWORD		GetImageXYPos();
	void		GetXYScale(XYSCALE FAR * AScale);
	virtual void 	Paint(HDC PaintDC, PAINTSTRUCT _FAR & PaintInfo);
	void		PromptDisplayOptions();
	void		PromptXYScale();
	void		SetDisplayOptions(DWORD ADisplayOptions);
	void		SetImageFile(LPSTR AImageFile);
	void		SetImageXYPos(int AXPos, int AYPos);
	void		SetInitValues();
	void		SetXYScale(XYSCALE FAR * AScale);
	void		UpdateScrollers();
	virtual void	WMGetDisplayOptions(RTMessage Msg) = [TKD_GETDISPLAYOPTIONS];
	virtual void	WMGetImageXYPos(RTMessage Msg)	= [TKD_GETIMAGEXYPOS];
	virtual void	WMGetXYScale(RTMessage Msg)	= [TKD_GETXYSCALE];
	virtual void 	WMLButtonDown(RTMessage Msg) 	= [WM_FIRST + WM_LBUTTONDOWN];
  	virtual void 	WMLButtonUp(RTMessage Msg) 	= [WM_FIRST + WM_LBUTTONUP];
	virtual void 	WMMouseMove(RTMessage Msg) 	= [WM_FIRST + WM_MOUSEMOVE];
	virtual void	WMPaint(RTMessage Msg)		= [WM_FIRST + WM_PAINT];
	virtual void	WMPromptDisplayOptions(RTMessage Msg) 	= [TKD_PROMPTDISPLAYOPTIONS];
	virtual void	WMPromptXYScale(RTMessage Msg) 	= [TKD_PROMPTXYSCALE];
	virtual void	WMSetDisplayOptions(RTMessage Msg)	= [TKD_SETDISPLAYOPTIONS];
	virtual void	WMSetImageFile(RTMessage Msg)	= [TKD_SETIMAGEFILE];
	virtual void	WMSetImageXYPos(RTMessage Msg)	= [TKD_SETIMAGEXYPOS];
	virtual void	WMSetXYScale(RTMessage Msg)	= [TKD_SETXYSCALE];
};

#endif // __TKOFAXW_H__
